@@ -1,8 +1,13 @@
+"""Python dataclass model generator.
+
+Generates Python dataclasses using the @dataclass decorator.
+"""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING, Any, ClassVar, Optional
 
-from datamodel_code_generator import DatetimeClassType, PythonVersion, PythonVersionMin
+from datamodel_code_generator import DataclassArguments, DatetimeClassType, PythonVersion, PythonVersionMin
 from datamodel_code_generator.imports import (
     IMPORT_DATE,
     IMPORT_DATETIME,
@@ -13,6 +18,7 @@ from datamodel_code_generator.imports import (
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model.base import UNDEFINED
 from datamodel_code_generator.model.imports import IMPORT_DATACLASS, IMPORT_FIELD
+from datamodel_code_generator.model.pydantic.base_model import Constraints  # noqa: TC001 # needed for pydantic
 from datamodel_code_generator.model.types import DataTypeManager as _DataTypeManager
 from datamodel_code_generator.model.types import type_map_factory
 from datamodel_code_generator.types import DataType, StrictTypes, Types, chain_as_tuple
@@ -24,16 +30,17 @@ if TYPE_CHECKING:
 
     from datamodel_code_generator.reference import Reference
 
-from datamodel_code_generator.model.pydantic.base_model import Constraints  # noqa: TC001
 
-
-def _has_field_assignment(field: DataModelFieldBase) -> bool:
+def has_field_assignment(field: DataModelFieldBase) -> bool:
+    """Check if a dataclass field has a default value or field() assignment."""
     return bool(field.field) or not (
         field.required or (field.represented_default == "None" and field.strip_default_none)
     )
 
 
 class DataClass(DataModel):
+    """DataModel implementation for Python dataclasses."""
+
     TEMPLATE_FILE_PATH: ClassVar[str] = "dataclass.jinja2"
     DEFAULT_IMPORTS: ClassVar[tuple[Import, ...]] = (IMPORT_DATACLASS,)
 
@@ -55,10 +62,12 @@ class DataClass(DataModel):
         keyword_only: bool = False,
         frozen: bool = False,
         treat_dot_as_module: bool = False,
+        dataclass_arguments: DataclassArguments | None = None,
     ) -> None:
+        """Initialize dataclass with fields sorted by field assignment requirement."""
         super().__init__(
             reference=reference,
-            fields=sorted(fields, key=_has_field_assignment),
+            fields=sorted(fields, key=has_field_assignment),
             decorators=decorators,
             base_classes=base_classes,
             custom_base_class=custom_base_class,
@@ -73,9 +82,19 @@ class DataClass(DataModel):
             frozen=frozen,
             treat_dot_as_module=treat_dot_as_module,
         )
+        if dataclass_arguments is not None:
+            self.dataclass_arguments = dataclass_arguments
+        else:
+            self.dataclass_arguments = {}
+            if frozen:
+                self.dataclass_arguments["frozen"] = True
+            if keyword_only:
+                self.dataclass_arguments["kw_only"] = True
 
 
 class DataModelField(DataModelFieldBase):
+    """Field implementation for dataclass models."""
+
     _FIELD_KEYS: ClassVar[set[str]] = {
         "default_factory",
         "init",
@@ -88,36 +107,27 @@ class DataModelField(DataModelFieldBase):
     constraints: Optional[Constraints] = None  # noqa: UP045
 
     def process_const(self) -> None:
-        if "const" not in self.extras:
-            return
-        self.const = True
-        self.nullable = False
-        const = self.extras["const"]
-        self.data_type = self.data_type.__class__(literals=[const])
-        if not self.default:
-            self.default = const
+        """Process const field constraint using literal type."""
+        self._process_const_as_literal()
 
     @property
     def imports(self) -> tuple[Import, ...]:
+        """Get imports including field() if needed."""
         field = self.field
         if field and field.startswith("field("):
             return chain_as_tuple(super().imports, (IMPORT_FIELD,))
         return super().imports
 
-    def self_reference(self) -> bool:  # pragma: no cover
-        return isinstance(self.parent, DataClass) and self.parent.reference.path in {
-            d.reference.path for d in self.data_type.all_data_types if d.reference
-        }
-
     @property
     def field(self) -> str | None:
-        """for backwards compatibility"""
+        """For backwards compatibility."""
         result = str(self)
         if not result:
             return None
         return result
 
     def __str__(self) -> str:
+        """Generate field() call or default value representation."""
         data: dict[str, Any] = {k: v for k, v in self.extras.items() if k in self._FIELD_KEYS}
 
         if self.default != UNDEFINED and self.default is not None:
@@ -140,14 +150,21 @@ class DataModelField(DataModelFieldBase):
         if len(data) == 1 and "default" in data:
             default = data["default"]
 
-            if isinstance(default, (list, dict)):
-                return f"field(default_factory=lambda :{default!r})"
+            if isinstance(default, (list, dict, set)):
+                if default:
+                    from datamodel_code_generator.model.base import repr_set_sorted  # noqa: PLC0415
+
+                    default_repr = repr_set_sorted(default) if isinstance(default, set) else repr(default)
+                    return f"field(default_factory=lambda: {default_repr})"
+                return f"field(default_factory={type(default).__name__})"
             return repr(default)
         kwargs = [f"{k}={v if k == 'default_factory' else repr(v)}" for k, v in data.items()]
         return f"field({', '.join(kwargs)})"
 
 
 class DataTypeManager(_DataTypeManager):
+    """Type manager for dataclass models."""
+
     def __init__(  # noqa: PLR0913, PLR0917
         self,
         python_version: PythonVersion = PythonVersionMin,
@@ -155,21 +172,26 @@ class DataTypeManager(_DataTypeManager):
         use_generic_container_types: bool = False,  # noqa: FBT001, FBT002
         strict_types: Sequence[StrictTypes] | None = None,
         use_non_positive_negative_number_constrained_types: bool = False,  # noqa: FBT001, FBT002
+        use_decimal_for_multiple_of: bool = False,  # noqa: FBT001, FBT002
         use_union_operator: bool = False,  # noqa: FBT001, FBT002
         use_pendulum: bool = False,  # noqa: FBT001, FBT002
         target_datetime_class: DatetimeClassType = DatetimeClassType.Datetime,
         treat_dot_as_module: bool = False,  # noqa: FBT001, FBT002
+        use_serialize_as_any: bool = False,  # noqa: FBT001, FBT002
     ) -> None:
+        """Initialize type manager with datetime type mapping."""
         super().__init__(
             python_version,
             use_standard_collections,
             use_generic_container_types,
             strict_types,
             use_non_positive_negative_number_constrained_types,
+            use_decimal_for_multiple_of,
             use_union_operator,
             use_pendulum,
             target_datetime_class,
             treat_dot_as_module,
+            use_serialize_as_any,
         )
 
         datetime_map = (
