@@ -1,3 +1,8 @@
+"""Import definitions for model modules.
+
+Provides pre-defined Import objects for dataclasses, TypedDict, msgspec, etc.
+"""
+
 from __future__ import annotations
 
 from datamodel_code_generator.imports import Import
@@ -13,3 +18,5 @@ IMPORT_MSGSPEC_STRUCT = Import.from_full_path("msgspec.Struct")
 IMPORT_MSGSPEC_FIELD = Import.from_full_path("msgspec.field")
 IMPORT_MSGSPEC_META = Import.from_full_path("msgspec.Meta")
 IMPORT_MSGSPEC_CONVERT = Import.from_full_path("msgspec.convert")
+IMPORT_MSGSPEC_UNSET = Import.from_full_path("msgspec.UNSET")
+IMPORT_MSGSPEC_UNSETTYPE = Import.from_full_path("msgspec.UnsetType")
