@@ -1,3 +1,9 @@
+"""Pydantic v2 model generator.
+
+Provides BaseModel, RootModel, and DataModelField for generating
+Pydantic v2 compatible data models with ConfigDict support.
+"""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING, Optional
@@ -13,10 +19,13 @@ if TYPE_CHECKING:
 
 
 def dump_resolve_reference_action(class_names: Iterable[str]) -> str:
+    """Generate model_rebuild() calls for Pydantic v2 models."""
     return "\n".join(f"{class_name}.model_rebuild()" for class_name in class_names)
 
 
 class ConfigDict(_BaseModel):
+    """Pydantic v2 model_config options."""
+
     extra: Optional[str] = None  # noqa: UP045
     title: Optional[str] = None  # noqa: UP045
     populate_by_name: Optional[bool] = None  # noqa: UP045
@@ -29,6 +38,7 @@ class ConfigDict(_BaseModel):
     regex_engine: Optional[str] = None  # noqa: UP045
     use_enum_values: Optional[bool] = None  # noqa: UP045
     coerce_numbers_to_str: Optional[bool] = None  # noqa: UP045
+    use_attribute_docstrings: Optional[bool] = None  # noqa: UP045
 
 
 __all__ = [
