@@ -1,3 +1,8 @@
+"""Pydantic v2 RootModel implementation.
+
+Generates models inheriting from pydantic.RootModel for wrapping single types.
+"""
+
 from __future__ import annotations
 
 from typing import Any, ClassVar, Literal
@@ -6,6 +11,8 @@ from datamodel_code_generator.model.pydantic_v2.base_model import BaseModel
 
 
 class RootModel(BaseModel):
+    """DataModel for Pydantic v2 RootModel."""
+
     TEMPLATE_FILE_PATH: ClassVar[str] = "pydantic_v2/RootModel.jinja2"
     BASE_CLASS: ClassVar[str] = "pydantic.RootModel"
 
@@ -13,8 +20,11 @@ class RootModel(BaseModel):
         self,
         **kwargs: Any,
     ) -> None:
-        # Remove custom_base_class for Pydantic V2 models; behaviour is different from Pydantic V1 as it will not
-        # be treated as a root model. custom_base_class cannot both implement BaseModel and RootModel!
+        """Initialize RootModel and remove custom_base_class if present.
+
+        Remove custom_base_class for Pydantic V2 models; behaviour is different from Pydantic V1 as it will not
+        be treated as a root model. custom_base_class cannot both implement BaseModel and RootModel!
+        """
         if "custom_base_class" in kwargs:
             kwargs.pop("custom_base_class")
 
