@@ -1,3 +1,8 @@
+"""TypedDict model generator.
+
+Generates Python TypedDict classes for use with type checkers.
+"""
+
 from __future__ import annotations
 
 import keyword
@@ -17,9 +22,9 @@ if TYPE_CHECKING:
     from collections.abc import Iterator
     from pathlib import Path
 
+    from datamodel_code_generator.imports import Import
     from datamodel_code_generator.reference import Reference
 
-from datamodel_code_generator.imports import Import  # noqa: TC001
 
 escape_characters = str.maketrans({
     "\\": r"\\",
@@ -40,6 +45,8 @@ def _is_valid_field_name(field: DataModelFieldBase) -> bool:
 
 
 class TypedDict(DataModel):
+    """DataModel implementation for Python TypedDict."""
+
     TEMPLATE_FILE_PATH: ClassVar[str] = "TypedDict.jinja2"
     BASE_CLASS: ClassVar[str] = "typing.TypedDict"
     DEFAULT_IMPORTS: ClassVar[tuple[Import, ...]] = (IMPORT_TYPED_DICT,)
@@ -62,6 +69,7 @@ class TypedDict(DataModel):
         keyword_only: bool = False,
         treat_dot_as_module: bool = False,
     ) -> None:
+        """Initialize TypedDict model."""
         super().__init__(
             reference=reference,
             fields=fields,
@@ -81,23 +89,16 @@ class TypedDict(DataModel):
 
     @property
     def is_functional_syntax(self) -> bool:
+        """Check if TypedDict requires functional syntax."""
         return any(not _is_valid_field_name(f) for f in self.fields)
 
     @property
     def all_fields(self) -> Iterator[DataModelFieldBase]:
-        for base_class in self.base_classes:
-            if base_class.reference is None:  # pragma: no cover
-                continue
-            data_model = base_class.reference.source
-            if not isinstance(data_model, DataModel):  # pragma: no cover
-                continue
-
-            if isinstance(data_model, TypedDict):  # pragma: no cover
-                yield from data_model.all_fields
-
-        yield from self.fields
+        """Iterate over all fields including inherited ones."""
+        yield from self.iter_all_fields()
 
     def render(self, *, class_name: str | None = None) -> str:
+        """Render TypedDict class with appropriate syntax."""
         return self._render(
             class_name=class_name or self.class_name,
             fields=self.fields,
@@ -112,26 +113,24 @@ class TypedDict(DataModel):
 
 
 class DataModelField(DataModelFieldBase):
+    """Field implementation for TypedDict models."""
+
     DEFAULT_IMPORTS: ClassVar[tuple[Import, ...]] = (IMPORT_NOT_REQUIRED,)
 
     def process_const(self) -> None:
-        if "const" not in self.extras:
-            return
-        self.const = True
-        self.nullable = False
-        const = self.extras["const"]
-        self.data_type = self.data_type.__class__(literals=[const])
-        if not self.default:
-            self.default = const
+        """Process const field constraint using literal type."""
+        self._process_const_as_literal()
 
     @property
     def key(self) -> str:
+        """Get escaped field key for TypedDict."""
         return (self.original_name or self.name or "").translate(  # pragma: no cover
             escape_characters
         )
 
     @property
     def type_hint(self) -> str:
+        """Get type hint with NotRequired wrapper if needed."""
         type_hint = super().type_hint
         if self._not_required:
             return f"{NOT_REQUIRED_PREFIX}{type_hint}]"
@@ -139,14 +138,17 @@ class DataModelField(DataModelFieldBase):
 
     @property
     def _not_required(self) -> bool:
+        """Check if field should be marked as NotRequired."""
         return not self.required and isinstance(self.parent, TypedDict)
 
     @property
     def fall_back_to_nullable(self) -> bool:
+        """Check if field should fall back to nullable."""
         return not self._not_required
 
     @property
     def imports(self) -> tuple[Import, ...]:
+        """Get imports including NotRequired if needed."""
         return (
             *super().imports,
             *(self.DEFAULT_IMPORTS if self._not_required else ()),
@@ -154,4 +156,6 @@ class DataModelField(DataModelFieldBase):
 
 
 class DataModelFieldBackport(DataModelField):
+    """Field implementation for TypedDict models using typing_extensions."""
+
     DEFAULT_IMPORTS: ClassVar[tuple[Import, ...]] = (IMPORT_NOT_REQUIRED_BACKPORT,)
