@@ -1,3 +1,9 @@
+"""Parser utilities and base types for schema parsing.
+
+Provides LiteralType enum for literal parsing options and DefaultPutDict
+for caching remote schema content.
+"""
+
 from __future__ import annotations
 
 from collections import UserDict
@@ -9,17 +15,22 @@ TV = TypeVar("TV")
 
 
 class LiteralType(Enum):
+    """Options for handling enum fields as literals."""
+
     All = "all"
     One = "one"
 
 
 class DefaultPutDict(UserDict[TK, TV]):
+    """Dict that can lazily compute and cache missing values."""
+
     def get_or_put(
         self,
         key: TK,
         default: TV | None = None,
         default_factory: Callable[[TK], TV] | None = None,
     ) -> TV:
+        """Get value for key, or compute and store it if missing."""
         if key in self:
             return self[key]
         if default:  # pragma: no cover
@@ -28,7 +39,7 @@ class DefaultPutDict(UserDict[TK, TV]):
         if default_factory:
             value = self[key] = default_factory(key)
             return value
-        msg = "Not found default and default_factory"
+        msg = "Not found default and default_factory"  # pragma: no cover
         raise ValueError(msg)  # pragma: no cover
 
 
