@@ -1,3 +1,9 @@
+"""GraphQL schema parser implementation.
+
+Parses GraphQL schema files to generate Python data models including
+objects, interfaces, enums, scalars, inputs, and union types.
+"""
+
 from __future__ import annotations
 
 from pathlib import Path
@@ -9,18 +15,24 @@ from typing import (
 from urllib.parse import ParseResult
 
 from datamodel_code_generator import (
+    DEFAULT_SHARED_MODULE_NAME,
+    AllOfMergeMode,
+    DataclassArguments,
     DefaultPutDict,
     LiteralType,
     PythonVersion,
     PythonVersionMin,
+    ReadOnlyWriteOnlyModelType,
+    ReuseScope,
     snooper_to_methods,
 )
+from datamodel_code_generator.format import DEFAULT_FORMATTERS, DatetimeClassType, Formatter
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model import pydantic as pydantic_model
 from datamodel_code_generator.model.dataclass import DataClass
-from datamodel_code_generator.model.enum import Enum
-from datamodel_code_generator.model.scalar import DataTypeScalar
-from datamodel_code_generator.model.union import DataTypeUnion
+from datamodel_code_generator.model.enum import SPECIALIZED_ENUM_TYPE_MATCH, Enum
+from datamodel_code_generator.model.scalar import DataTypeScalarBackport
+from datamodel_code_generator.model.union import DataTypeUnionBackport
 from datamodel_code_generator.parser.base import (
     DataType,
     Parser,
@@ -37,13 +49,16 @@ except ImportError as exc:  # pragma: no cover
     raise Exception(msg) from exc  # noqa: TRY002
 
 
-from datamodel_code_generator.format import DEFAULT_FORMATTERS, DatetimeClassType, Formatter
-
 if TYPE_CHECKING:
     from collections import defaultdict
     from collections.abc import Iterable, Iterator, Mapping, Sequence
 
-graphql_resolver = graphql.type.introspection.TypeResolvers()
+# graphql-core >=3.2.7 removed TypeResolvers in favor of TypeFields.kind.
+# Normalize to a single callable for resolving type kinds.
+try:  # graphql-core < 3.2.7
+    graphql_resolver_kind = graphql.type.introspection.TypeResolvers().kind  # pyright: ignore[reportAttributeAccessIssue]
+except AttributeError:  # pragma: no cover - executed on newer graphql-core
+    graphql_resolver_kind = graphql.type.introspection.TypeFields.kind  # pyright: ignore[reportAttributeAccessIssue]
 
 
 def build_graphql_schema(schema_str: str) -> graphql.GraphQLSchema:
@@ -54,6 +69,8 @@ def build_graphql_schema(schema_str: str) -> graphql.GraphQLSchema:
 
 @snooper_to_methods()
 class GraphQLParser(Parser):
+    """Parser for GraphQL schema files."""
+
     # raw graphql schema as `graphql-core` object
     raw_obj: graphql.GraphQLSchema
     # all processed graphql objects
@@ -84,8 +101,8 @@ class GraphQLParser(Parser):
         *,
         data_model_type: type[DataModel] = pydantic_model.BaseModel,
         data_model_root_type: type[DataModel] = pydantic_model.CustomRootType,
-        data_model_scalar_type: type[DataModel] = DataTypeScalar,
-        data_model_union_type: type[DataModel] = DataTypeUnion,
+        data_model_scalar_type: type[DataModel] = DataTypeScalarBackport,
+        data_model_union_type: type[DataModel] = DataTypeUnionBackport,
         data_type_manager_type: type[DataTypeManager] = pydantic_model.DataTypeManager,
         data_model_field_type: type[DataModelFieldBase] = pydantic_model.DataModelField,
         base_class: str | None = None,
@@ -109,12 +126,17 @@ class GraphQLParser(Parser):
         base_path: Path | None = None,
         use_schema_description: bool = False,
         use_field_description: bool = False,
+        use_attribute_docstrings: bool = False,
+        use_inline_field_description: bool = False,
         use_default_kwarg: bool = False,
         reuse_model: bool = False,
+        reuse_scope: ReuseScope | None = None,
+        shared_module_name: str = DEFAULT_SHARED_MODULE_NAME,
         encoding: str = "utf-8",
         enum_field_as_literal: LiteralType | None = None,
         set_default_enum_member: bool = False,
         use_subclass_enum: bool = False,
+        use_specialized_enum: bool = True,
         strict_nullable: bool = False,
         use_generic_container_types: bool = False,
         enable_faux_immutability: bool = False,
@@ -130,20 +152,25 @@ class GraphQLParser(Parser):
         use_title_as_name: bool = False,
         use_operation_id_as_name: bool = False,
         use_unique_items_as_set: bool = False,
+        allof_merge_mode: AllOfMergeMode = AllOfMergeMode.Constraints,
         http_headers: Sequence[tuple[str, str]] | None = None,
         http_ignore_tls: bool = False,
         use_annotated: bool = False,
         use_non_positive_negative_number_constrained_types: bool = False,
+        use_decimal_for_multiple_of: bool = False,
         original_field_name_delimiter: str | None = None,
         use_double_quotes: bool = False,
         use_union_operator: bool = False,
         allow_responses_without_content: bool = False,
         collapse_root_models: bool = False,
+        skip_root_model: bool = False,
+        use_type_alias: bool = False,
         special_field_name_prefix: str | None = None,
         remove_special_field_name_prefix: bool = False,
         capitalise_enum_members: bool = False,
         keep_model_order: bool = False,
         use_one_literal_as_default: bool = False,
+        use_enum_values_in_discriminator: bool = False,
         known_third_party: list[str] | None = None,
         custom_formatters: list[str] | None = None,
         custom_formatters_kwargs: dict[str, Any] | None = None,
@@ -158,7 +185,13 @@ class GraphQLParser(Parser):
         no_alias: bool = False,
         formatters: list[Formatter] = DEFAULT_FORMATTERS,
         parent_scoped_naming: bool = False,
+        dataclass_arguments: DataclassArguments | None = None,
+        type_mappings: list[str] | None = None,
+        read_only_write_only_model_type: ReadOnlyWriteOnlyModelType | None = None,
+        use_serialize_as_any: bool = False,
+        use_frozen_field: bool = False,
     ) -> None:
+        """Initialize the GraphQL parser with configuration options."""
         super().__init__(
             source=source,
             data_model_type=data_model_type,
@@ -186,13 +219,19 @@ class GraphQLParser(Parser):
             base_path=base_path,
             use_schema_description=use_schema_description,
             use_field_description=use_field_description,
+            use_attribute_docstrings=use_attribute_docstrings,
+            use_inline_field_description=use_inline_field_description,
             use_default_kwarg=use_default_kwarg,
             reuse_model=reuse_model,
+            reuse_scope=reuse_scope,
+            shared_module_name=shared_module_name,
             encoding=encoding,
             enum_field_as_literal=enum_field_as_literal,
             use_one_literal_as_default=use_one_literal_as_default,
+            use_enum_values_in_discriminator=use_enum_values_in_discriminator,
             set_default_enum_member=set_default_enum_member,
             use_subclass_enum=use_subclass_enum,
+            use_specialized_enum=use_specialized_enum,
             strict_nullable=strict_nullable,
             use_generic_container_types=use_generic_container_types,
             enable_faux_immutability=enable_faux_immutability,
@@ -208,15 +247,19 @@ class GraphQLParser(Parser):
             use_title_as_name=use_title_as_name,
             use_operation_id_as_name=use_operation_id_as_name,
             use_unique_items_as_set=use_unique_items_as_set,
+            allof_merge_mode=allof_merge_mode,
             http_headers=http_headers,
             http_ignore_tls=http_ignore_tls,
             use_annotated=use_annotated,
             use_non_positive_negative_number_constrained_types=use_non_positive_negative_number_constrained_types,
+            use_decimal_for_multiple_of=use_decimal_for_multiple_of,
             original_field_name_delimiter=original_field_name_delimiter,
             use_double_quotes=use_double_quotes,
             use_union_operator=use_union_operator,
             allow_responses_without_content=allow_responses_without_content,
             collapse_root_models=collapse_root_models,
+            skip_root_model=skip_root_model,
+            use_type_alias=use_type_alias,
             special_field_name_prefix=special_field_name_prefix,
             remove_special_field_name_prefix=remove_special_field_name_prefix,
             capitalise_enum_members=capitalise_enum_members,
@@ -235,6 +278,11 @@ class GraphQLParser(Parser):
             no_alias=no_alias,
             formatters=formatters,
             parent_scoped_naming=parent_scoped_naming,
+            dataclass_arguments=dataclass_arguments,
+            type_mappings=type_mappings,
+            read_only_write_only_model_type=read_only_write_only_model_type,
+            use_serialize_as_any=use_serialize_as_any,
+            use_frozen_field=use_frozen_field,
         )
 
         self.data_model_scalar_type = data_model_scalar_type
@@ -275,7 +323,7 @@ class GraphQLParser(Parser):
             if type_name in {"Query", "Mutation"}:
                 continue
 
-            resolved_type = graphql_resolver.kind(type_, None)
+            resolved_type = graphql_resolver_kind(type_, None)
 
             if resolved_type in self.support_graphql_types:  # pragma: no cover
                 self.all_graphql_objects[type_.name] = type_
@@ -289,10 +337,26 @@ class GraphQLParser(Parser):
                 self.support_graphql_types[resolved_type].append(type_)
 
     def _create_data_model(self, model_type: type[DataModel] | None = None, **kwargs: Any) -> DataModel:
-        """Create data model instance with conditional frozen parameter for DataClass."""
+        """Create data model instance with dataclass_arguments support for DataClass."""
         data_model_class = model_type or self.data_model_type
         if issubclass(data_model_class, DataClass):
-            kwargs["frozen"] = self.frozen_dataclasses
+            # Use dataclass_arguments from kwargs, or fall back to self.dataclass_arguments
+            # If both are None, construct from legacy frozen_dataclasses/keyword_only flags
+            dataclass_arguments = kwargs.pop("dataclass_arguments", None)
+            if dataclass_arguments is None:
+                dataclass_arguments = self.dataclass_arguments
+            if dataclass_arguments is None:
+                # Construct from legacy flags for library API compatibility
+                dataclass_arguments = {}
+                if self.frozen_dataclasses:
+                    dataclass_arguments["frozen"] = True
+                if self.keyword_only:
+                    dataclass_arguments["kw_only"] = True
+            kwargs["dataclass_arguments"] = dataclass_arguments
+            kwargs.pop("frozen", None)
+            kwargs.pop("keyword_only", None)
+        else:
+            kwargs.pop("dataclass_arguments", None)
         return data_model_class(**kwargs)
 
     def _typename_field(self, name: str) -> DataModelFieldBase:
@@ -316,7 +380,8 @@ class GraphQLParser(Parser):
         self,
         field: graphql.GraphQLField | graphql.GraphQLInputField,
         final_data_type: DataType,
-        required: bool,  # noqa: FBT001
+        *,
+        required: bool,
     ) -> Any:
         if isinstance(field, graphql.GraphQLInputField):  # pragma: no cover
             if field.default_value == graphql.pyutils.Undefined:  # pragma: no cover
@@ -328,6 +393,7 @@ class GraphQLParser(Parser):
         return None
 
     def parse_scalar(self, scalar_graphql_object: graphql.GraphQLScalarType) -> None:
+        """Parse a GraphQL scalar type and add it to results."""
         self.results.append(
             self.data_model_scalar_type(
                 reference=self.references[scalar_graphql_object.name],
@@ -338,7 +404,40 @@ class GraphQLParser(Parser):
             )
         )
 
+    def should_parse_enum_as_literal(self, obj: graphql.GraphQLEnumType) -> bool:
+        """Determine if an enum should be parsed as a literal type."""
+        return self.enum_field_as_literal == LiteralType.All or (
+            self.enum_field_as_literal == LiteralType.One and len(obj.values) == 1
+        )
+
     def parse_enum(self, enum_object: graphql.GraphQLEnumType) -> None:
+        """Parse a GraphQL enum type and add it to results."""
+        if self.should_parse_enum_as_literal(enum_object):
+            return self.parse_enum_as_literal(enum_object)
+        return self.parse_enum_as_enum_class(enum_object)
+
+    def parse_enum_as_literal(self, enum_object: graphql.GraphQLEnumType) -> None:
+        """Parse enum values as a Literal type."""
+        data_type = self.data_type(literals=list(enum_object.values.keys()))
+        data_model_type = self._create_data_model(
+            model_type=self.data_model_root_type,
+            reference=self.references[enum_object.name],
+            fields=[
+                self.data_model_field_type(
+                    required=True,
+                    data_type=data_type,
+                )
+            ],
+            custom_base_class=self.base_class,
+            custom_template_dir=self.custom_template_dir,
+            extra_template_data=self.extra_template_data,
+            path=self.current_source_path,
+            description=enum_object.description,
+        )
+        self.results.append(data_model_type)
+
+    def parse_enum_as_enum_class(self, enum_object: graphql.GraphQLEnumType) -> None:
+        """Parse enum values as an Enum class."""
         enum_fields: list[DataModelFieldBase] = []
         exclude_field_names: set[str] = set()
 
@@ -365,11 +464,21 @@ class GraphQLParser(Parser):
                 )
             )
 
-        enum = Enum(
+        enum_cls: type[Enum] = Enum
+        if (
+            self.target_python_version.has_strenum
+            and self.use_specialized_enum
+            and (specialized_type := SPECIALIZED_ENUM_TYPE_MATCH.get(Types.string))
+        ):
+            # If specialized enum is available in the target Python version, use it
+            enum_cls = specialized_type
+
+        enum: Enum = enum_cls(
             reference=self.references[enum_object.name],
             fields=enum_fields,
             path=self.current_source_path,
             description=enum_object.description,
+            type_=Types.string if self.use_subclass_enum else None,
             custom_template_dir=self.custom_template_dir,
         )
         self.results.append(enum)
@@ -380,6 +489,7 @@ class GraphQLParser(Parser):
         alias: str | None,
         field: graphql.GraphQLField | graphql.GraphQLInputField,
     ) -> DataModelFieldBase:
+        """Parse a GraphQL field and return a data model field."""
         final_data_type = DataType(
             is_optional=True,
             use_union_operator=self.use_union_operator,
@@ -406,16 +516,16 @@ class GraphQLParser(Parser):
             obj = graphql.assert_wrapping_type(obj)
             obj = obj.of_type
 
-        if graphql.is_enum_type(obj):
-            obj = graphql.assert_enum_type(obj)
-            data_type.reference = self.references[obj.name]
-
         obj = graphql.assert_named_type(obj)
-        data_type.type = obj.name
+        if obj.name in self.references:
+            data_type.reference = self.references[obj.name]
+        else:  # pragma: no cover
+            # Only happens for Query and Mutation root types
+            data_type.type = obj.name
 
         required = (not self.force_optional_for_required_fields) and (not final_data_type.is_optional)
 
-        default = self._get_default(field, final_data_type, required)
+        default = self._get_default(field, final_data_type, required=required)
         extras = {} if self.default_field_extras is None else self.default_field_extras.copy()
 
         if field.description is not None:  # pragma: no cover
@@ -430,7 +540,9 @@ class GraphQLParser(Parser):
             alias=alias,
             strip_default_none=self.strip_default_none,
             use_annotated=self.use_annotated,
+            use_serialize_as_any=self.use_serialize_as_any,
             use_field_description=self.use_field_description,
+            use_inline_field_description=self.use_inline_field_description,
             use_default_kwarg=self.use_default_kwarg,
             original_name=field_name,
             has_default=default is not None,
@@ -440,12 +552,16 @@ class GraphQLParser(Parser):
         self,
         obj: graphql.GraphQLInterfaceType | graphql.GraphQLObjectType | graphql.GraphQLInputObjectType,
     ) -> None:
+        """Parse a GraphQL object-like type and add it to results."""
         fields = []
         exclude_field_names: set[str] = set()
 
         for field_name, field in obj.fields.items():
             field_name_, alias = self.model_resolver.get_valid_field_name_and_alias(
-                field_name, excludes=exclude_field_names
+                field_name,
+                excludes=exclude_field_names,
+                model_type=self.field_name_model_type,
+                class_name=obj.name,
             )
             exclude_field_names.add(field_name_)
 
@@ -469,19 +585,24 @@ class GraphQLParser(Parser):
             description=obj.description,
             keyword_only=self.keyword_only,
             treat_dot_as_module=self.treat_dot_as_module,
+            dataclass_arguments=self.dataclass_arguments,
         )
         self.results.append(data_model_type)
 
     def parse_interface(self, interface_graphql_object: graphql.GraphQLInterfaceType) -> None:
+        """Parse a GraphQL interface type and add it to results."""
         self.parse_object_like(interface_graphql_object)
 
     def parse_object(self, graphql_object: graphql.GraphQLObjectType) -> None:
+        """Parse a GraphQL object type and add it to results."""
         self.parse_object_like(graphql_object)
 
     def parse_input_object(self, input_graphql_object: graphql.GraphQLInputObjectType) -> None:
+        """Parse a GraphQL input object type and add it to results."""
         self.parse_object_like(input_graphql_object)  # pragma: no cover
 
     def parse_union(self, union_object: graphql.GraphQLUnionType) -> None:
+        """Parse a GraphQL union type and add it to results."""
         fields = [self.data_model_field_type(name=type_.name, data_type=DataType()) for type_ in union_object.types]
 
         data_model_type = self.data_model_union_type(
@@ -496,6 +617,7 @@ class GraphQLParser(Parser):
         self.results.append(data_model_type)
 
     def parse_raw(self) -> None:
+        """Parse the raw GraphQL schema and generate all data models."""
         self.all_graphql_objects = {}
         self.references: dict[str, Reference] = {}
 
