@@ -1,8 +1,16 @@
+"""OpenAPI and Swagger specification parser.
+
+Extends JsonSchemaParser to handle OpenAPI 2.0 (Swagger), 3.0, and 3.1
+specifications, including paths, operations, parameters, and request/response bodies.
+"""
+
 from __future__ import annotations
 
 import re
 from collections import defaultdict
+from contextlib import nullcontext
 from enum import Enum
+from pathlib import Path
 from re import Pattern
 from typing import TYPE_CHECKING, Any, Callable, ClassVar, Optional, TypeVar, Union
 from warnings import warn
@@ -10,25 +18,30 @@ from warnings import warn
 from pydantic import Field
 
 from datamodel_code_generator import (
+    DEFAULT_SHARED_MODULE_NAME,
+    AllOfMergeMode,
+    DataclassArguments,
     Error,
     LiteralType,
     OpenAPIScope,
     PythonVersion,
     PythonVersionMin,
-    load_yaml,
+    ReadOnlyWriteOnlyModelType,
+    ReuseScope,
+    YamlValue,
+    load_yaml_dict,
     snooper_to_methods,
 )
 from datamodel_code_generator.format import DEFAULT_FORMATTERS, DatetimeClassType, Formatter
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model import pydantic as pydantic_model
-from datamodel_code_generator.parser import DefaultPutDict  # noqa: TC001 # needed for type check
 from datamodel_code_generator.parser.base import get_special_path
 from datamodel_code_generator.parser.jsonschema import (
     JsonSchemaObject,
     JsonSchemaParser,
     get_model_by_path,
 )
-from datamodel_code_generator.reference import snake_to_upper_camel
+from datamodel_code_generator.reference import FieldNameResolver, is_url, snake_to_upper_camel
 from datamodel_code_generator.types import (
     DataType,
     DataTypeManager,
@@ -39,9 +52,10 @@ from datamodel_code_generator.util import BaseModel
 
 if TYPE_CHECKING:
     from collections.abc import Iterable, Mapping, Sequence
-    from pathlib import Path
     from urllib.parse import ParseResult
 
+    from datamodel_code_generator.parser import DefaultPutDict
+
 
 RE_APPLICATION_JSON_PATTERN: Pattern[str] = re.compile(r"^application/.*json$")
 
@@ -58,6 +72,8 @@ OPERATION_NAMES: list[str] = [
 
 
 class ParameterLocation(Enum):
+    """Represent OpenAPI parameter locations."""
+
     query = "query"
     header = "header"
     path = "path"
@@ -68,57 +84,73 @@ BaseModelT = TypeVar("BaseModelT", bound=BaseModel)
 
 
 class ReferenceObject(BaseModel):
+    """Represent an OpenAPI reference object ($ref)."""
+
     ref: str = Field(..., alias="$ref")
 
 
 class ExampleObject(BaseModel):
+    """Represent an OpenAPI example object."""
+
     summary: Optional[str] = None  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
-    value: Any = None
+    value: YamlValue = None
     externalValue: Optional[str] = None  # noqa: N815, UP045
 
 
 class MediaObject(BaseModel):
+    """Represent an OpenAPI media type object."""
+
     schema_: Optional[Union[ReferenceObject, JsonSchemaObject]] = Field(None, alias="schema")  # noqa: UP007, UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
 
 
 class ParameterObject(BaseModel):
+    """Represent an OpenAPI parameter object."""
+
     name: Optional[str] = None  # noqa: UP045
     in_: Optional[ParameterLocation] = Field(None, alias="in")  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
     required: bool = False
     deprecated: bool = False
     schema_: Optional[JsonSchemaObject] = Field(None, alias="schema")  # noqa: UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
 
 
 class HeaderObject(BaseModel):
+    """Represent an OpenAPI header object."""
+
     description: Optional[str] = None  # noqa: UP045
     required: bool = False
     deprecated: bool = False
     schema_: Optional[JsonSchemaObject] = Field(None, alias="schema")  # noqa: UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
 
 
 class RequestBodyObject(BaseModel):
+    """Represent an OpenAPI request body object."""
+
     description: Optional[str] = None  # noqa: UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
     required: bool = False
 
 
 class ResponseObject(BaseModel):
+    """Represent an OpenAPI response object."""
+
     description: Optional[str] = None  # noqa: UP045
     headers: dict[str, ParameterObject] = {}  # noqa: RUF012
     content: dict[Union[str, int], MediaObject] = {}  # noqa: RUF012, UP007
 
 
 class Operation(BaseModel):
+    """Represent an OpenAPI operation object."""
+
     tags: list[str] = []  # noqa: RUF012
     summary: Optional[str] = None  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
@@ -130,6 +162,8 @@ class Operation(BaseModel):
 
 
 class ComponentsObject(BaseModel):
+    """Represent an OpenAPI components object."""
+
     schemas: dict[str, Union[ReferenceObject, JsonSchemaObject]] = {}  # noqa: RUF012, UP007
     responses: dict[str, Union[ReferenceObject, ResponseObject]] = {}  # noqa: RUF012, UP007
     examples: dict[str, Union[ReferenceObject, ExampleObject]] = {}  # noqa: RUF012, UP007
@@ -139,6 +173,8 @@ class ComponentsObject(BaseModel):
 
 @snooper_to_methods()
 class OpenAPIParser(JsonSchemaParser):
+    """Parser for OpenAPI 2.0/3.0/3.1 and Swagger specifications."""
+
     SCHEMA_PATHS: ClassVar[list[str]] = ["#/components/schemas"]
 
     def __init__(  # noqa: PLR0913
@@ -170,13 +206,19 @@ class OpenAPIParser(JsonSchemaParser):
         base_path: Path | None = None,
         use_schema_description: bool = False,
         use_field_description: bool = False,
+        use_attribute_docstrings: bool = False,
+        use_inline_field_description: bool = False,
         use_default_kwarg: bool = False,
         reuse_model: bool = False,
+        reuse_scope: ReuseScope | None = None,
+        shared_module_name: str = DEFAULT_SHARED_MODULE_NAME,
         encoding: str = "utf-8",
         enum_field_as_literal: LiteralType | None = None,
         use_one_literal_as_default: bool = False,
+        use_enum_values_in_discriminator: bool = False,
         set_default_enum_member: bool = False,
         use_subclass_enum: bool = False,
+        use_specialized_enum: bool = True,
         strict_nullable: bool = False,
         use_generic_container_types: bool = False,
         enable_faux_immutability: bool = False,
@@ -194,15 +236,20 @@ class OpenAPIParser(JsonSchemaParser):
         use_title_as_name: bool = False,
         use_operation_id_as_name: bool = False,
         use_unique_items_as_set: bool = False,
+        allof_merge_mode: AllOfMergeMode = AllOfMergeMode.Constraints,
         http_headers: Sequence[tuple[str, str]] | None = None,
         http_ignore_tls: bool = False,
         use_annotated: bool = False,
+        use_serialize_as_any: bool = False,
         use_non_positive_negative_number_constrained_types: bool = False,
+        use_decimal_for_multiple_of: bool = False,
         original_field_name_delimiter: str | None = None,
         use_double_quotes: bool = False,
         use_union_operator: bool = False,
         allow_responses_without_content: bool = False,
         collapse_root_models: bool = False,
+        skip_root_model: bool = False,
+        use_type_alias: bool = False,
         special_field_name_prefix: str | None = None,
         remove_special_field_name_prefix: bool = False,
         capitalise_enum_members: bool = False,
@@ -221,7 +268,12 @@ class OpenAPIParser(JsonSchemaParser):
         no_alias: bool = False,
         formatters: list[Formatter] = DEFAULT_FORMATTERS,
         parent_scoped_naming: bool = False,
+        dataclass_arguments: DataclassArguments | None = None,
+        type_mappings: list[str] | None = None,
+        read_only_write_only_model_type: ReadOnlyWriteOnlyModelType | None = None,
+        use_frozen_field: bool = False,
     ) -> None:
+        """Initialize the OpenAPI parser with extensive configuration options."""
         target_datetime_class = target_datetime_class or DatetimeClassType.Awaredatetime
         super().__init__(
             source=source,
@@ -250,13 +302,19 @@ class OpenAPIParser(JsonSchemaParser):
             base_path=base_path,
             use_schema_description=use_schema_description,
             use_field_description=use_field_description,
+            use_attribute_docstrings=use_attribute_docstrings,
+            use_inline_field_description=use_inline_field_description,
             use_default_kwarg=use_default_kwarg,
             reuse_model=reuse_model,
+            reuse_scope=reuse_scope,
+            shared_module_name=shared_module_name,
             encoding=encoding,
             enum_field_as_literal=enum_field_as_literal,
             use_one_literal_as_default=use_one_literal_as_default,
+            use_enum_values_in_discriminator=use_enum_values_in_discriminator,
             set_default_enum_member=set_default_enum_member,
             use_subclass_enum=use_subclass_enum,
+            use_specialized_enum=use_specialized_enum,
             strict_nullable=strict_nullable,
             use_generic_container_types=use_generic_container_types,
             enable_faux_immutability=enable_faux_immutability,
@@ -272,15 +330,20 @@ class OpenAPIParser(JsonSchemaParser):
             use_title_as_name=use_title_as_name,
             use_operation_id_as_name=use_operation_id_as_name,
             use_unique_items_as_set=use_unique_items_as_set,
+            allof_merge_mode=allof_merge_mode,
             http_headers=http_headers,
             http_ignore_tls=http_ignore_tls,
             use_annotated=use_annotated,
+            use_serialize_as_any=use_serialize_as_any,
             use_non_positive_negative_number_constrained_types=use_non_positive_negative_number_constrained_types,
+            use_decimal_for_multiple_of=use_decimal_for_multiple_of,
             original_field_name_delimiter=original_field_name_delimiter,
             use_double_quotes=use_double_quotes,
             use_union_operator=use_union_operator,
             allow_responses_without_content=allow_responses_without_content,
             collapse_root_models=collapse_root_models,
+            skip_root_model=skip_root_model,
+            use_type_alias=use_type_alias,
             special_field_name_prefix=special_field_name_prefix,
             remove_special_field_name_prefix=remove_special_field_name_prefix,
             capitalise_enum_members=capitalise_enum_members,
@@ -299,16 +362,24 @@ class OpenAPIParser(JsonSchemaParser):
             no_alias=no_alias,
             formatters=formatters,
             parent_scoped_naming=parent_scoped_naming,
+            dataclass_arguments=dataclass_arguments,
+            type_mappings=type_mappings,
+            read_only_write_only_model_type=read_only_write_only_model_type,
+            use_frozen_field=use_frozen_field,
         )
         self.open_api_scopes: list[OpenAPIScope] = openapi_scopes or [OpenAPIScope.Schemas]
         self.include_path_parameters: bool = include_path_parameters
+        self._discriminator_schemas: dict[str, dict[str, Any]] = {}
+        self._discriminator_subtypes: dict[str, list[str]] = defaultdict(list)
 
     def get_ref_model(self, ref: str) -> dict[str, Any]:
+        """Resolve a reference to its model definition."""
         ref_file, ref_path = self.model_resolver.resolve_ref(ref).split("#", 1)
         ref_body = self._get_ref_body(ref_file) if ref_file else self.raw_obj
         return get_model_by_path(ref_body, ref_path.split("/")[1:])
 
     def get_data_type(self, obj: JsonSchemaObject) -> DataType:
+        """Get data type from JSON schema object, handling OpenAPI nullable semantics."""
         # OpenAPI 3.0 doesn't allow `null` in the `type` field and list of types
         # https://swagger.io/docs/specification/data-models/data-types/#null
         # OpenAPI 3.1 does allow `null` in the `type` field and is equivalent to
@@ -318,18 +389,125 @@ class OpenAPIParser(JsonSchemaParser):
 
         return super().get_data_type(obj)
 
+    def _get_discriminator_union_type(self, ref: str) -> DataType | None:
+        """Create a union type for discriminator subtypes if available."""
+        subtypes = self._discriminator_subtypes.get(ref, [])
+        if not subtypes:
+            return None
+        refs = map(self.model_resolver.add_ref, subtypes)
+        return self.data_type(data_types=[self.data_type(reference=r) for r in refs])
+
+    def get_ref_data_type(self, ref: str) -> DataType:
+        """Get data type for a reference, handling discriminator polymorphism."""
+        if ref in self._discriminator_schemas and (union_type := self._get_discriminator_union_type(ref)):
+            return union_type
+        return super().get_ref_data_type(ref)
+
+    def parse_object_fields(
+        self,
+        obj: JsonSchemaObject,
+        path: list[str],
+        module_name: Optional[str] = None,  # noqa: UP045
+        class_name: Optional[str] = None,  # noqa: UP045
+    ) -> list[DataModelFieldBase]:
+        """Parse object fields, adding discriminator info for allOf polymorphism."""
+        fields = super().parse_object_fields(obj, path, module_name, class_name=class_name)
+        properties = obj.properties or {}
+
+        result_fields: list[DataModelFieldBase] = []
+        for field_obj in fields:
+            field = properties.get(field_obj.original_name)
+
+            if (
+                isinstance(field, JsonSchemaObject)
+                and field.ref
+                and (discriminator := self._discriminator_schemas.get(field.ref))
+            ):
+                new_field_type = self._get_discriminator_union_type(field.ref) or field_obj.data_type
+                field_obj = self.data_model_field_type(**{  # noqa: PLW2901
+                    **field_obj.__dict__,
+                    "data_type": new_field_type,
+                    "extras": {**field_obj.extras, "discriminator": discriminator},
+                })
+            result_fields.append(field_obj)
+
+        return result_fields
+
     def resolve_object(self, obj: ReferenceObject | BaseModelT, object_type: type[BaseModelT]) -> BaseModelT:
+        """Resolve a reference object to its actual type or return the object as-is."""
         if isinstance(obj, ReferenceObject):
             ref_obj = self.get_ref_model(obj.ref)
             return object_type.parse_obj(ref_obj)
         return obj
 
+    def _parse_schema_or_ref(
+        self,
+        name: str,
+        schema: JsonSchemaObject | ReferenceObject | None,
+        path: list[str],
+    ) -> DataType | None:
+        """Parse a schema object or resolve a reference to get DataType."""
+        if schema is None:
+            return None
+        if isinstance(schema, JsonSchemaObject):
+            return self.parse_schema(name, schema, path)
+        self.resolve_ref(schema.ref)
+        return self.get_ref_data_type(schema.ref)
+
+    def _process_path_items(  # noqa: PLR0913
+        self,
+        items: dict[str, dict[str, Any]],
+        base_path: list[str],
+        scope_name: str,
+        global_parameters: list[dict[str, Any]],
+        security: list[dict[str, list[str]]] | None,
+        *,
+        strip_leading_slash: bool = True,
+    ) -> None:
+        """Process path or webhook items with operations."""
+        scope_path = [*base_path, f"#/{scope_name}"]
+        for item_name, methods_ in items.items():
+            item_ref = methods_.get("$ref")
+            if item_ref:
+                methods = self.get_ref_model(item_ref)
+                # Extract base path from reference for external file resolution
+                resolved_ref = self.model_resolver.resolve_ref(item_ref)
+                ref_file = resolved_ref.split("#")[0] if "#" in resolved_ref else resolved_ref
+                ref_base_path = Path(ref_file).parent if ref_file and not is_url(ref_file) else None
+            else:
+                methods = methods_
+                ref_base_path = None
+
+            item_parameters = global_parameters.copy()
+            if "parameters" in methods:
+                item_parameters.extend(methods["parameters"])
+
+            relative_name = item_name[1:] if strip_leading_slash else item_name.removeprefix("/")
+            path = [*scope_path, relative_name] if relative_name else get_special_path("root", scope_path)
+
+            base_path_context = (
+                self.model_resolver.current_base_path_context(ref_base_path) if ref_base_path else nullcontext()
+            )
+            with base_path_context:
+                for operation_name, raw_operation in methods.items():
+                    if operation_name not in OPERATION_NAMES:
+                        continue
+                    if item_parameters:
+                        if "parameters" in raw_operation:
+                            raw_operation["parameters"].extend(item_parameters)
+                        else:
+                            raw_operation["parameters"] = item_parameters.copy()
+                    if security is not None and "security" not in raw_operation:
+                        raw_operation["security"] = security
+                    self.parse_operation(raw_operation, [*path, operation_name])
+
     def parse_schema(
         self,
         name: str,
         obj: JsonSchemaObject,
         path: list[str],
     ) -> DataType:
+        """Parse a JSON schema object into a data type."""
         if obj.is_array:
             data_type = self.parse_array(name, obj, [*path, name])
         elif obj.allOf:  # pragma: no cover
@@ -355,15 +533,12 @@ class OpenAPIParser(JsonSchemaParser):
         request_body: RequestBodyObject,
         path: list[str],
     ) -> dict[str, DataType]:
+        """Parse request body content into data types by media type."""
         data_types: dict[str, DataType] = {}
-        for (
-            media_type,
-            media_obj,
-        ) in request_body.content.items():
-            if isinstance(media_obj.schema_, JsonSchemaObject):
-                data_types[media_type] = self.parse_schema(name, media_obj.schema_, [*path, media_type])
-            elif media_obj.schema_ is not None:
-                data_types[media_type] = self.get_ref_data_type(media_obj.schema_.ref)
+        for media_type, media_obj in request_body.content.items():
+            data_type = self._parse_schema_or_ref(name, media_obj.schema_, [*path, media_type])
+            if data_type:
+                data_types[media_type] = data_type
         return data_types
 
     def parse_responses(
@@ -372,6 +547,7 @@ class OpenAPIParser(JsonSchemaParser):
         responses: dict[str | int, ReferenceObject | ResponseObject],
         path: list[str],
     ) -> dict[str | int, dict[str, DataType]]:
+        """Parse response objects into data types by status code and content type."""
         data_types: defaultdict[str | int, dict[str, DataType]] = defaultdict(dict)
         for status_code, detail in responses.items():
             if isinstance(detail, ReferenceObject):
@@ -386,19 +562,10 @@ class OpenAPIParser(JsonSchemaParser):
                 data_types[status_code]["application/json"] = DataType(type="None")
 
             for content_type, obj in content.items():
-                object_schema = obj.schema_
-                if not object_schema:  # pragma: no cover
-                    continue
-                if isinstance(object_schema, JsonSchemaObject):
-                    data_types[status_code][content_type] = self.parse_schema(  # pyright: ignore[reportArgumentType]
-                        name,
-                        object_schema,
-                        [*path, str(status_code), content_type],  # pyright: ignore[reportArgumentType]
-                    )
-                else:
-                    data_types[status_code][content_type] = self.get_ref_data_type(  # pyright: ignore[reportArgumentType]
-                        object_schema.ref
-                    )
+                response_path: list[str] = [*path, str(status_code), str(content_type)]
+                data_type = self._parse_schema_or_ref(name, obj.schema_, response_path)
+                if data_type:
+                    data_types[status_code][content_type] = data_type  # pyright: ignore[reportArgumentType]
 
         return data_types
 
@@ -409,11 +576,15 @@ class OpenAPIParser(JsonSchemaParser):
         tags: list[str],
         path: list[str],  # noqa: ARG003
     ) -> list[str]:
+        """Parse operation tags."""
         return tags
 
+    _field_name_resolver: FieldNameResolver = FieldNameResolver()
+
     @classmethod
     def _get_model_name(cls, path_name: str, method: str, suffix: str) -> str:
-        camel_path_name = snake_to_upper_camel(path_name.replace("/", "_"))
+        normalized = cls._field_name_resolver.get_valid_name(path_name, ignore_snake_case_field=True)
+        camel_path_name = snake_to_upper_camel(normalized)
         return f"{camel_path_name}{method.capitalize()}{suffix}"
 
     def parse_all_parameters(
@@ -422,6 +593,7 @@ class OpenAPIParser(JsonSchemaParser):
         parameters: list[ReferenceObject | ParameterObject],
         path: list[str],
     ) -> DataType | None:
+        """Parse all operation parameters into a data model."""
         fields: list[DataModelFieldBase] = []
         exclude_field_names: set[str] = set()
         reference = self.model_resolver.add(path, name, class_name=True, unique=True)
@@ -440,7 +612,10 @@ class OpenAPIParser(JsonSchemaParser):
                 raise Exception(msg)  # noqa: TRY002
 
             field_name, alias = self.model_resolver.get_valid_field_name_and_alias(
-                field_name=parameter_name, excludes=exclude_field_names
+                field_name=parameter_name,
+                excludes=exclude_field_names,
+                model_type=self.field_name_model_type,
+                class_name=name,
             )
             if parameter.schema_:
                 fields.append(
@@ -495,7 +670,9 @@ class OpenAPIParser(JsonSchemaParser):
                         strip_default_none=self.strip_default_none,
                         extras=self.get_field_extras(object_schema) if object_schema else {},
                         use_annotated=self.use_annotated,
+                        use_serialize_as_any=self.use_serialize_as_any,
                         use_field_description=self.use_field_description,
+                        use_inline_field_description=self.use_inline_field_description,
                         use_default_kwarg=self.use_default_kwarg,
                         original_name=parameter_name,
                         has_default=object_schema.has_default if object_schema else False,
@@ -514,6 +691,7 @@ class OpenAPIParser(JsonSchemaParser):
                     custom_template_dir=self.custom_template_dir,
                     keyword_only=self.keyword_only,
                     treat_dot_as_module=self.treat_dot_as_module,
+                    dataclass_arguments=self.dataclass_arguments,
                 )
             )
             return self.data_type(reference=reference)
@@ -525,6 +703,7 @@ class OpenAPIParser(JsonSchemaParser):
         raw_operation: dict[str, Any],
         path: list[str],
     ) -> None:
+        """Parse an OpenAPI operation including parameters, request body, and responses."""
         operation = Operation.parse_obj(raw_operation)
         path_name, method = path[-2:]
         if self.use_operation_id_as_name:
@@ -566,8 +745,9 @@ class OpenAPIParser(JsonSchemaParser):
                 path=[*path, "tags"],
             )
 
-    def parse_raw(self) -> None:  # noqa: PLR0912
-        for source, path_parts in self._get_context_source_path_parts():  # noqa: PLR1702
+    def parse_raw(self) -> None:
+        """Parse OpenAPI specification including schemas, paths, and operations."""
+        for source, path_parts in self._get_context_source_path_parts():
             if self.validation:
                 warn(
                     "Deprecated: `--validation` option is deprecated. the option will be removed in a future "
@@ -592,52 +772,59 @@ class OpenAPIParser(JsonSchemaParser):
                         stacklevel=2,
                     )
 
-            specification: dict[str, Any] = load_yaml(source.text)
+            specification: dict[str, Any] = load_yaml_dict(source.text)
             self.raw_obj = specification
-            schemas: dict[Any, Any] = specification.get("components", {}).get("schemas", {})
+            self._collect_discriminator_schemas()
+            schemas: dict[str, Any] = specification.get("components", {}).get("schemas", {})
+            paths: dict[str, Any] = specification.get("paths", {})
             security: list[dict[str, list[str]]] | None = specification.get("security")
+            # Warn if schemas is empty but paths exist and only Schemas scope is used
+            if not schemas and self.open_api_scopes == [OpenAPIScope.Schemas] and paths:
+                warn(
+                    "No schemas found in components/schemas. If your schemas are defined in "
+                    "external files referenced from paths, consider using --openapi-scopes paths",
+                    stacklevel=2,
+                )
             if OpenAPIScope.Schemas in self.open_api_scopes:
-                for (
-                    obj_name,
-                    raw_obj,
-                ) in schemas.items():
+                for obj_name, raw_obj in schemas.items():
                     self.parse_raw_obj(
                         obj_name,
                         raw_obj,
                         [*path_parts, "#/components", "schemas", obj_name],
                     )
             if OpenAPIScope.Paths in self.open_api_scopes:
-                paths: dict[str, dict[str, Any]] = specification.get("paths", {})
-                parameters: list[dict[str, Any]] = [
-                    self._get_ref_body(p["$ref"]) if "$ref" in p else p
+                # Resolve $ref in global parameter list
+                global_parameters = [
+                    self._get_ref_body(p["$ref"]) if isinstance(p, dict) and "$ref" in p else p
                     for p in paths.get("parameters", [])
                     if isinstance(p, dict)
                 ]
-                paths_path = [*path_parts, "#/paths"]
-                for path_name, methods_ in paths.items():
-                    # Resolve path items if applicable
-                    methods = self.get_ref_model(methods_["$ref"]) if "$ref" in methods_ else methods_
-                    paths_parameters = parameters.copy()
-                    if "parameters" in methods:
-                        paths_parameters.extend(methods["parameters"])
-                    relative_path_name = path_name[1:]
-                    if relative_path_name:
-                        path = [*paths_path, relative_path_name]
-                    else:  # pragma: no cover
-                        path = get_special_path("root", paths_path)
-                    for operation_name, raw_operation in methods.items():
-                        if operation_name not in OPERATION_NAMES:
-                            continue
-                        if paths_parameters:
-                            if "parameters" in raw_operation:  # pragma: no cover
-                                raw_operation["parameters"].extend(paths_parameters)
-                            else:
-                                raw_operation["parameters"] = paths_parameters
-                        if security is not None and "security" not in raw_operation:
-                            raw_operation["security"] = security
-                        self.parse_operation(
-                            raw_operation,
-                            [*path, operation_name],
-                        )
+                self._process_path_items(paths, path_parts, "paths", global_parameters, security)
+
+            if OpenAPIScope.Webhooks in self.open_api_scopes:
+                webhooks: dict[str, dict[str, Any]] = specification.get("webhooks", {})
+                self._process_path_items(webhooks, path_parts, "webhooks", [], security, strip_leading_slash=False)
 
         self._resolve_unparsed_json_pointer()
+
+    def _collect_discriminator_schemas(self) -> None:
+        """Collect schemas with discriminators but no oneOf/anyOf, and find their subtypes."""
+        schemas: dict[str, Any] = self.raw_obj.get("components", {}).get("schemas", {})
+
+        for schema_name, schema in schemas.items():
+            discriminator = schema.get("discriminator")
+            if not discriminator:
+                continue
+
+            if schema.get("oneOf") or schema.get("anyOf"):
+                continue
+
+            ref = f"#/components/schemas/{schema_name}"
+            self._discriminator_schemas[ref] = discriminator
+
+        for schema_name, schema in schemas.items():
+            for all_of_item in schema.get("allOf", []):
+                ref_in_allof = all_of_item.get("$ref")
+                if ref_in_allof and ref_in_allof in self._discriminator_schemas:
+                    subtype_ref = f"#/components/schemas/{schema_name}"
+                    self._discriminator_subtypes[ref_in_allof].append(subtype_ref)
