@@ -1,3 +1,10 @@
+"""Reference resolution and model tracking system.
+
+Provides Reference for tracking model references across schemas, ModelResolver
+for managing class names and field names, and FieldNameResolver for converting
+schema field names to valid Python identifiers.
+"""
+
 from __future__ import annotations
 
 import re
@@ -9,24 +16,72 @@ from itertools import zip_longest
 from keyword import iskeyword
 from pathlib import Path, PurePath
 from re import Pattern
-from typing import TYPE_CHECKING, Any, Callable, ClassVar, NamedTuple, Optional, TypeVar
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    ClassVar,
+    NamedTuple,
+    Optional,
+    Protocol,
+    TypeVar,
+    cast,
+    runtime_checkable,
+)
 from urllib.parse import ParseResult, urlparse
 
-import inflect
 import pydantic
 from packaging import version
-from pydantic import BaseModel
+from pydantic import BaseModel, Field
+from typing_extensions import TypeIs
 
-from datamodel_code_generator.util import PYDANTIC_V2, ConfigDict, model_validator
+from datamodel_code_generator import Error
+from datamodel_code_generator.util import PYDANTIC_V2, ConfigDict, camel_to_snake, model_validator
 
 if TYPE_CHECKING:
-    from collections.abc import Generator, Mapping, Sequence
+    from collections.abc import Generator, Iterator, Mapping, Sequence
     from collections.abc import Set as AbstractSet
 
+    import inflect
     from pydantic.typing import DictStrAny
 
+    from datamodel_code_generator.model.base import DataModel
+    from datamodel_code_generator.types import DataType
+
+
+def _is_data_type(value: object) -> TypeIs[DataType]:
+    """Check if value is a DataType instance."""
+    from datamodel_code_generator.types import DataType as DataType_  # noqa: PLC0415
+
+    return isinstance(value, DataType_)
+
+
+def _is_data_model(value: object) -> TypeIs[DataModel]:
+    """Check if value is a DataModel instance."""
+    from datamodel_code_generator.model.base import DataModel as DataModel_  # noqa: PLC0415
+
+    return isinstance(value, DataModel_)
+
+
+@runtime_checkable
+class ReferenceChild(Protocol):
+    """Protocol for objects that can be stored in Reference.children.
+
+    This is a minimal protocol - actual usage checks isinstance for DataType
+    or DataModel to access specific methods like replace_reference or class_name.
+    Using a property makes the type covariant, allowing both DataModel (Reference)
+    and DataType (Reference | None) to satisfy this protocol.
+    """
+
+    @property
+    def reference(self) -> Reference | None:
+        """Return the reference associated with this object."""
+        ...
+
 
 class _BaseModel(BaseModel):
+    """Base model with field exclusion and pass-through support."""
+
     _exclude_fields: ClassVar[set[str]] = set()
     _pass_fields: ClassVar[set[str]] = set()
 
@@ -85,20 +140,23 @@ class _BaseModel(BaseModel):
 
 
 class Reference(_BaseModel):
+    """Represents a reference to a model in the schema.
+
+    Tracks path, name, and relationships between models for resolution.
+    """
+
     path: str
     original_name: str = ""
     name: str
     duplicate_name: Optional[str] = None  # noqa: UP045
     loaded: bool = True
-    source: Optional[Any] = None  # noqa: UP045
-    children: list[Any] = []
+    source: Optional[ReferenceChild] = None  # noqa: UP045
+    children: list[ReferenceChild] = Field(default_factory=list)
     _exclude_fields: ClassVar[set[str]] = {"children"}
 
     @model_validator(mode="before")
     def validate_original_name(cls, values: Any) -> Any:  # noqa: N805
-        """
-        If original_name is empty then, `original_name` is assigned `name`
-        """
+        """Assign name to original_name if original_name is empty."""
         if not isinstance(values, dict):  # pragma: no cover
             return values
         original_name = values.get("original_name")
@@ -119,24 +177,42 @@ class Reference(_BaseModel):
     else:
 
         class Config:
+            """Pydantic v1 configuration for Reference model."""
+
             arbitrary_types_allowed = True
             keep_untouched = (cached_property,)
             copy_on_model_validation = False if version.parse(pydantic.VERSION) < version.parse("1.9.2") else "none"
 
     @property
     def short_name(self) -> str:
+        """Return the last component of the dotted name."""
         return self.name.rsplit(".", 1)[-1]
 
+    def replace_children_references(self, new_reference: Reference) -> None:
+        """Replace all DataType children's reference with new_reference."""
+        for child in self.children[:]:
+            if _is_data_type(child):
+                child.replace_reference(new_reference)
+
+    def iter_data_model_children(self) -> Iterator[DataModel]:
+        """Yield all DataModel children."""
+        for child in self.children:
+            if _is_data_model(child):
+                yield child
+
 
 SINGULAR_NAME_SUFFIX: str = "Item"
 
 ID_PATTERN: Pattern[str] = re.compile(r"^#[^/].*")
 
+SPECIAL_PATH_MARKER: str = "#-datamodel-code-generator-#-"
+
 T = TypeVar("T")
 
 
 @contextmanager
 def context_variable(setter: Callable[[T], None], current_value: T, new_value: T) -> Generator[None, None, None]:
+    """Context manager that temporarily sets a value and restores it on exit."""
     previous_value: T = current_value
     setter(new_value)
     try:
@@ -145,17 +221,9 @@ def context_variable(setter: Callable[[T], None], current_value: T, new_value: T
         setter(previous_value)
 
 
-_UNDER_SCORE_1: Pattern[str] = re.compile(r"([^_])([A-Z][a-z]+)")
-_UNDER_SCORE_2: Pattern[str] = re.compile(r"([a-z0-9])([A-Z])")
-
-
-@lru_cache
-def camel_to_snake(string: str) -> str:
-    subbed = _UNDER_SCORE_1.sub(r"\1_\2", string)
-    return _UNDER_SCORE_2.sub(r"\1_\2", subbed).lower()
-
-
 class FieldNameResolver:
+    """Converts schema field names to valid Python identifiers."""
+
     def __init__(  # noqa: PLR0913, PLR0917
         self,
         aliases: Mapping[str, str] | None = None,
@@ -167,6 +235,7 @@ class FieldNameResolver:
         capitalise_enum_members: bool = False,  # noqa: FBT001, FBT002
         no_alias: bool = False,  # noqa: FBT001, FBT002
     ) -> None:
+        """Initialize field name resolver with transformation options."""
         self.aliases: Mapping[str, str] = {} if aliases is None else {**aliases}
         self.empty_field_name: str = empty_field_name or "_"
         self.snake_case_field = snake_case_field
@@ -180,6 +249,7 @@ class FieldNameResolver:
 
     @classmethod
     def _validate_field_name(cls, field_name: str) -> bool:  # noqa: ARG003
+        """Check if a field name is valid. Subclasses may override."""
         return True
 
     def get_valid_name(  # noqa: PLR0912
@@ -189,6 +259,7 @@ class FieldNameResolver:
         ignore_snake_case_field: bool = False,  # noqa: FBT001, FBT002
         upper_camel: bool = False,  # noqa: FBT001, FBT002
     ) -> str:
+        """Convert a name to a valid Python identifier."""
         if not name:
             name = self.empty_field_name
         if name[0] == "#":
@@ -221,19 +292,43 @@ class FieldNameResolver:
         else:
             new_name = name
         while (
-            not (new_name.isidentifier() or not self._validate_field_name(new_name))
+            not new_name.isidentifier()
             or iskeyword(new_name)
             or (excludes and new_name in excludes)
+            or not self._validate_field_name(new_name)
         ):
             new_name = f"{name}{count}" if upper_camel else f"{name}_{count}"
             count += 1
         return new_name
 
     def get_valid_field_name_and_alias(
-        self, field_name: str, excludes: set[str] | None = None
+        self,
+        field_name: str,
+        excludes: set[str] | None = None,
+        path: list[str] | None = None,
+        class_name: str | None = None,
     ) -> tuple[str, str | None]:
+        """Get valid field name and original alias if different.
+
+        Supports hierarchical alias resolution with the following priority:
+        1. Scoped aliases (ClassName.field_name) - class-level specificity
+        2. Flat aliases (field_name) - applies to all occurrences
+
+        Args:
+            field_name: The original field name from the schema.
+            excludes: Set of names to avoid when generating valid names.
+            path: Unused, kept for backward compatibility.
+            class_name: Optional class name for scoped alias resolution.
+        """
+        del path
+        if class_name:
+            scoped_key = f"{class_name}.{field_name}"
+            if scoped_key in self.aliases:
+                return self.aliases[scoped_key], field_name
+
         if field_name in self.aliases:
             return self.aliases[field_name], field_name
+
         valid_name = self.get_valid_name(field_name, excludes=excludes)
         return (
             valid_name,
@@ -242,13 +337,103 @@ class FieldNameResolver:
 
 
 class PydanticFieldNameResolver(FieldNameResolver):
+    """Field name resolver that avoids Pydantic reserved names."""
+
     @classmethod
     def _validate_field_name(cls, field_name: str) -> bool:
+        """Check field name doesn't conflict with BaseModel attributes."""
         # TODO: Support Pydantic V2
         return not hasattr(BaseModel, field_name)
 
 
 class EnumFieldNameResolver(FieldNameResolver):
+    """Field name resolver for enum members with special handling for reserved names.
+
+    When using --use-subclass-enum, enums inherit from types like str or int.
+    Member names that conflict with methods of these types cause type checker errors.
+    This class detects and handles such conflicts by adding underscore suffixes.
+
+    The _BUILTIN_TYPE_ATTRIBUTES set is intentionally static (not using hasattr)
+    to avoid runtime Python version differences affecting code generation.
+    Based on Python 3.8-3.14 method names (union of all versions for safety).
+    Note: 'mro' is handled explicitly in get_valid_name for backward compatibility.
+    """
+
+    _BUILTIN_TYPE_ATTRIBUTES: ClassVar[frozenset[str]] = frozenset({
+        "as_integer_ratio",
+        "bit_count",
+        "bit_length",
+        "capitalize",
+        "casefold",
+        "center",
+        "conjugate",
+        "count",
+        "decode",
+        "denominator",
+        "encode",
+        "endswith",
+        "expandtabs",
+        "find",
+        "format",
+        "format_map",
+        "from_bytes",
+        "from_number",
+        "fromhex",
+        "hex",
+        "imag",
+        "index",
+        "isalnum",
+        "isalpha",
+        "isascii",
+        "isdecimal",
+        "isdigit",
+        "isidentifier",
+        "islower",
+        "isnumeric",
+        "isprintable",
+        "isspace",
+        "istitle",
+        "isupper",
+        "is_integer",
+        "join",
+        "ljust",
+        "lower",
+        "lstrip",
+        "maketrans",
+        "numerator",
+        "partition",
+        "real",
+        "removeprefix",
+        "removesuffix",
+        "replace",
+        "rfind",
+        "rindex",
+        "rjust",
+        "rpartition",
+        "rsplit",
+        "rstrip",
+        "split",
+        "splitlines",
+        "startswith",
+        "strip",
+        "swapcase",
+        "title",
+        "to_bytes",
+        "translate",
+        "upper",
+        "zfill",
+    })
+
+    @classmethod
+    def _validate_field_name(cls, field_name: str) -> bool:
+        """Check field name doesn't conflict with subclass enum base type attributes.
+
+        When using --use-subclass-enum, enums inherit from types like str or int.
+        Member names that conflict with methods of these types (e.g., 'count' for str)
+        cause type checker errors. This method detects such conflicts.
+        """
+        return field_name not in cls._BUILTIN_TYPE_ATTRIBUTES
+
     def get_valid_name(
         self,
         name: str,
@@ -256,6 +441,7 @@ class EnumFieldNameResolver(FieldNameResolver):
         ignore_snake_case_field: bool = False,  # noqa: FBT001, FBT002
         upper_camel: bool = False,  # noqa: FBT001, FBT002
     ) -> str:
+        """Convert name to valid enum member, handling reserved names."""
         return super().get_valid_name(
             name="mro_" if name == "mro" else name,
             excludes={"mro"} | (excludes or set()),
@@ -265,6 +451,8 @@ class EnumFieldNameResolver(FieldNameResolver):
 
 
 class ModelType(Enum):
+    """Type of model for field name resolution strategy."""
+
     PYDANTIC = auto()
     ENUM = auto()
     CLASS = auto()
@@ -278,11 +466,14 @@ DEFAULT_FIELD_NAME_RESOLVERS: dict[ModelType, type[FieldNameResolver]] = {
 
 
 class ClassName(NamedTuple):
+    """A class name with optional duplicate name for disambiguation."""
+
     name: str
     duplicate_name: str | None
 
 
 def get_relative_path(base_path: PurePath, target_path: PurePath) -> PurePath:
+    """Calculate relative path from base to target."""
     if base_path == target_path:
         return Path()
     if not target_path.is_absolute():
@@ -300,6 +491,12 @@ def get_relative_path(base_path: PurePath, target_path: PurePath) -> PurePath:
 
 
 class ModelResolver:  # noqa: PLR0904
+    """Manages model references, class names, and field name resolution.
+
+    Central registry for all model references during parsing, handling
+    name uniqueness, path resolution, and field name transformations.
+    """
+
     def __init__(  # noqa: PLR0913, PLR0917
         self,
         exclude_names: set[str] | None = None,
@@ -319,7 +516,9 @@ class ModelResolver:  # noqa: PLR0904
         no_alias: bool = False,  # noqa: FBT001, FBT002
         remove_suffix_number: bool = False,  # noqa: FBT001, FBT002
         parent_scoped_naming: bool = False,  # noqa: FBT001, FBT002
+        treat_dot_as_module: bool = False,  # noqa: FBT001, FBT002
     ) -> None:
+        """Initialize model resolver with naming and resolution options."""
         self.references: dict[str, Reference] = {}
         self._current_root: Sequence[str] = []
         self._root_id: str | None = None
@@ -353,31 +552,45 @@ class ModelResolver:  # noqa: PLR0904
         self._current_base_path: Path | None = self._base_path
         self.remove_suffix_number: bool = remove_suffix_number
         self.parent_scoped_naming = parent_scoped_naming
+        self.treat_dot_as_module = treat_dot_as_module
 
     @property
     def current_base_path(self) -> Path | None:
+        """Return the current base path for file resolution."""
         return self._current_base_path
 
     def set_current_base_path(self, base_path: Path | None) -> None:
+        """Set the current base path for file resolution."""
         self._current_base_path = base_path
 
     @property
     def base_url(self) -> str | None:
+        """Return the base URL for reference resolution."""
         return self._base_url
 
     def set_base_url(self, base_url: str | None) -> None:
+        """Set the base URL for reference resolution."""
         self._base_url = base_url
 
     @contextmanager
     def current_base_path_context(self, base_path: Path | None) -> Generator[None, None, None]:
+        """Temporarily set the current base path within a context."""
         if base_path:
             base_path = (self._base_path / base_path).resolve()
         with context_variable(self.set_current_base_path, self.current_base_path, base_path):
             yield
 
     @contextmanager
-    def base_url_context(self, base_url: str) -> Generator[None, None, None]:
-        if self._base_url:
+    def base_url_context(self, base_url: str | None) -> Generator[None, None, None]:
+        """Temporarily set the base URL within a context.
+
+        Only sets the base_url if:
+        - The new value is actually a URL (http://, https://, or file://)
+        - OR _base_url was already set (switching between URLs)
+        This preserves backward compatibility for local file parsing where
+        this method was previously a no-op.
+        """
+        if self._base_url or (base_url and is_url(base_url)):
             with context_variable(self.set_base_url, self.base_url, base_url):
                 yield
         else:
@@ -385,27 +598,31 @@ class ModelResolver:  # noqa: PLR0904
 
     @property
     def current_root(self) -> Sequence[str]:
-        if len(self._current_root) > 1:
-            return self._current_root
+        """Return the current root path components."""
         return self._current_root
 
     def set_current_root(self, current_root: Sequence[str]) -> None:
+        """Set the current root path components."""
         self._current_root = current_root
 
     @contextmanager
     def current_root_context(self, current_root: Sequence[str]) -> Generator[None, None, None]:
+        """Temporarily set the current root path within a context."""
         with context_variable(self.set_current_root, self.current_root, current_root):
             yield
 
     @property
     def root_id(self) -> str | None:
+        """Return the root identifier for the current schema."""
         return self._root_id
 
     @property
     def root_id_base_path(self) -> str | None:
+        """Return the base path component of the root identifier."""
         return self._root_id_base_path
 
     def set_root_id(self, root_id: str | None) -> None:
+        """Set the root identifier and extract its base path."""
         if root_id and "/" in root_id:
             self._root_id_base_path = root_id.rsplit("/", 1)[0]
         else:
@@ -414,37 +631,63 @@ class ModelResolver:  # noqa: PLR0904
         self._root_id = root_id
 
     def add_id(self, id_: str, path: Sequence[str]) -> None:
+        """Register an identifier mapping to a resolved reference path."""
         self.ids["/".join(self.current_root)][id_] = self.resolve_ref(path)
 
-    def resolve_ref(self, path: Sequence[str] | str) -> str:  # noqa: PLR0911, PLR0912
+    def resolve_ref(self, path: Sequence[str] | str) -> str:  # noqa: PLR0911, PLR0912, PLR0914
+        """Resolve a reference path to its canonical form."""
         joined_path = path if isinstance(path, str) else self.join_path(path)
         if joined_path == "#":
             return f"{'/'.join(self.current_root)}#"
         if self.current_base_path and not self.base_url and joined_path[0] != "#" and not is_url(joined_path):
             # resolve local file path
-            file_path, *object_part = joined_path.split("#", 1)
+            file_path, fragment = joined_path.split("#", 1) if "#" in joined_path else (joined_path, "")
             resolved_file_path = Path(self.current_base_path, file_path).resolve()
             joined_path = get_relative_path(self._base_path, resolved_file_path).as_posix()
-            if object_part:
-                joined_path += f"#{object_part[0]}"
-        if ID_PATTERN.match(joined_path):
-            ref: str = self.ids["/".join(self.current_root)][joined_path]
+            if fragment:
+                joined_path += f"#{fragment}"
+        if ID_PATTERN.match(joined_path) and SPECIAL_PATH_MARKER not in joined_path:
+            id_scope = "/".join(self.current_root)
+            scoped_ids = self.ids[id_scope]
+            ref: str | None = scoped_ids.get(joined_path)
+            if ref is None:
+                msg = (
+                    f"Unresolved $id reference '{joined_path}' in scope '{id_scope or '<root>'}'. "
+                    f"Known $id values: {', '.join(sorted(scoped_ids)) or '<none>'}"
+                )
+                raise Error(msg)
         else:
             if "#" not in joined_path:
                 joined_path += "#"
-            elif joined_path[0] == "#":
+            elif joined_path[0] == "#" and self.current_root:
                 joined_path = f"{'/'.join(self.current_root)}{joined_path}"
 
-            delimiter = joined_path.index("#")
-            file_path = "".join(joined_path[:delimiter])
-            ref = f"{''.join(joined_path[:delimiter])}#{''.join(joined_path[delimiter + 1 :])}"
-            if self.root_id_base_path and not (is_url(joined_path) or Path(self._base_path, file_path).is_file()):
+            file_path, fragment = joined_path.split("#", 1)
+            ref = f"{file_path}#{fragment}"
+            if (
+                self.root_id_base_path
+                and not self.base_url
+                and not (is_url(joined_path) or Path(self._base_path, file_path).is_file())
+            ):
                 ref = f"{self.root_id_base_path}/{ref}"
 
+        if is_url(ref):
+            file_part, path_part = ref.split("#", 1)
+            id_scope = "/".join(self.current_root)
+            scoped_ids = self.ids[id_scope]
+            if file_part in scoped_ids:
+                mapped_ref = scoped_ids[file_part]
+                if path_part:
+                    mapped_base, mapped_fragment = mapped_ref.split("#", 1) if "#" in mapped_ref else (mapped_ref, "")
+                    combined_fragment = f"{mapped_fragment.rstrip('/')}/{path_part.lstrip('/')}"
+                    return f"{mapped_base}#{combined_fragment}"
+                return mapped_ref
+
         if self.base_url:
             from .http import join_url  # noqa: PLC0415
 
-            joined_url = join_url(self.base_url, ref)
+            effective_base = self.root_id or self.base_url
+            joined_url = join_url(effective_base, ref)
             if "#" in joined_url:
                 return joined_url
             return f"{joined_url}#"
@@ -462,14 +705,18 @@ class ModelResolver:  # noqa: PLR0904
                 root_id_url.netloc,
             ):  # pragma: no cover
                 target_url_path = Path(target_url.path)
-                relative_target_base = get_relative_path(Path(root_id_url.path).parent, target_url_path.parent)
-                target_path = self.current_base_path / relative_target_base / target_url_path.name
+                target_path = (
+                    self.current_base_path
+                    / get_relative_path(Path(root_id_url.path).parent, target_url_path.parent)
+                    / target_url_path.name
+                )
                 if target_path.exists():
                     return f"{target_path.resolve().relative_to(self._base_path)}#{path_part}"
 
         return ref
 
     def is_after_load(self, ref: str) -> bool:
+        """Check if a reference points to a file loaded after the current one."""
         if is_url(ref) or not self.current_base_path:
             return False
         file_part, *_ = ref.split("#", 1)
@@ -480,23 +727,26 @@ class ModelResolver:  # noqa: PLR0904
 
     @staticmethod
     def is_external_ref(ref: str) -> bool:
+        """Check if a reference points to an external file."""
         return "#" in ref and ref[0] != "#"
 
     @staticmethod
     def is_external_root_ref(ref: str) -> bool:
-        return ref[-1] == "#"
+        """Check if a reference points to an external file root."""
+        return bool(ref) and ref[-1] == "#"
 
     @staticmethod
     def join_path(path: Sequence[str]) -> str:
+        """Join path components with slashes and normalize anchors."""
         joined_path = "/".join(p for p in path if p).replace("/#", "#")
         if "#" not in joined_path:
             joined_path += "#"
         return joined_path
 
     def add_ref(self, ref: str, resolved: bool = False) -> Reference:  # noqa: FBT001, FBT002
+        """Add a reference and return the Reference object."""
         path = self.resolve_ref(ref) if not resolved else ref
-        reference = self.references.get(path)
-        if reference:
+        if reference := self.references.get(path):
             return reference
         split_ref = ref.rsplit("/", 1)
         if len(split_ref) == 1:
@@ -516,15 +766,11 @@ class ModelResolver:  # noqa: PLR0904
 
     def _check_parent_scope_option(self, name: str, path: Sequence[str]) -> str:
         if self.parent_scoped_naming:
-            parent_reference = None
             parent_path = path[:-1]
             while parent_path:
-                parent_reference = self.references.get(self.join_path(parent_path))
-                if parent_reference is not None:
-                    break
+                if parent_reference := self.references.get(self.join_path(parent_path)):
+                    return f"{parent_reference.name}_{name}"
                 parent_path = parent_path[:-1]
-            if parent_reference:
-                name = f"{parent_reference.name}_{name}"
         return name
 
     def add(  # noqa: PLR0913
@@ -538,6 +784,7 @@ class ModelResolver:  # noqa: PLR0904
         singular_name_suffix: str | None = None,
         loaded: bool = False,
     ) -> Reference:
+        """Add or update a model reference with the given path and name."""
         joined_path = self.join_path(path)
         reference: Reference | None = self.references.get(joined_path)
         if reference:
@@ -563,7 +810,7 @@ class ModelResolver:  # noqa: PLR0904
                 name = get_singular_name(name, singular_name_suffix or self.singular_name_suffix)
             elif unique:  # pragma: no cover
                 unique_name = self._get_unique_name(name)
-                if unique_name == name:
+                if unique_name != name:
                     duplicate_name = name
                 name = unique_name
         if reference:
@@ -583,13 +830,17 @@ class ModelResolver:  # noqa: PLR0904
         return reference
 
     def get(self, path: Sequence[str] | str) -> Reference | None:
+        """Get a reference by path, returning None if not found."""
         return self.references.get(self.resolve_ref(path))
 
     def delete(self, path: Sequence[str] | str) -> None:
-        if self.resolve_ref(path) in self.references:
-            del self.references[self.resolve_ref(path)]
+        """Delete a reference by path if it exists."""
+        resolved = self.resolve_ref(path)
+        if resolved in self.references:
+            del self.references[resolved]
 
     def default_class_name_generator(self, name: str) -> str:
+        """Generate a valid class name from a string."""
         # TODO: create a validate for class name
         return self.field_name_resolvers[ModelType.CLASS].get_valid_name(
             name, ignore_snake_case_field=True, upper_camel=True
@@ -603,6 +854,7 @@ class ModelResolver:  # noqa: PLR0904
         singular_name: bool = False,  # noqa: FBT001, FBT002
         singular_name_suffix: str | None = None,
     ) -> ClassName:
+        """Generate a unique class name with optional singularization."""
         if "." in name:
             split_name = name.split(".")
             prefix = ".".join(
@@ -651,6 +903,7 @@ class ModelResolver:  # noqa: PLR0904
 
     @classmethod
     def validate_name(cls, name: str) -> bool:
+        """Check if a name is a valid Python identifier."""
         return name.isidentifier() and not iskeyword(name)
 
     def get_valid_field_name(
@@ -659,6 +912,7 @@ class ModelResolver:  # noqa: PLR0904
         excludes: set[str] | None = None,
         model_type: ModelType = ModelType.PYDANTIC,
     ) -> str:
+        """Get a valid field name for the specified model type."""
         return self.field_name_resolvers[model_type].get_valid_name(name, excludes)
 
     def get_valid_field_name_and_alias(
@@ -666,13 +920,44 @@ class ModelResolver:  # noqa: PLR0904
         field_name: str,
         excludes: set[str] | None = None,
         model_type: ModelType = ModelType.PYDANTIC,
+        path: list[str] | None = None,
+        class_name: str | None = None,
     ) -> tuple[str, str | None]:
-        return self.field_name_resolvers[model_type].get_valid_field_name_and_alias(field_name, excludes)
+        """Get a valid field name and alias for the specified model type.
+
+        Args:
+            field_name: The original field name from the schema.
+            excludes: Set of names to avoid when generating valid names.
+            model_type: The type of model (PYDANTIC, ENUM, or CLASS).
+            path: Unused, kept for backward compatibility.
+            class_name: Optional class name for scoped alias resolution.
+
+        Returns:
+            A tuple of (valid_field_name, alias_or_none).
+        """
+        del path
+        return self.field_name_resolvers[model_type].get_valid_field_name_and_alias(
+            field_name, excludes, class_name=class_name
+        )
+
+
+def _get_inflect_engine() -> inflect.engine:
+    """Get or create the inflect engine lazily."""
+    global _inflect_engine  # noqa: PLW0603
+    if _inflect_engine is None:
+        import inflect  # noqa: PLC0415
+
+        _inflect_engine = inflect.engine()
+    return _inflect_engine
+
+
+_inflect_engine: inflect.engine | None = None
 
 
 @lru_cache
 def get_singular_name(name: str, suffix: str = SINGULAR_NAME_SUFFIX) -> str:
-    singular_name = inflect_engine.singular_noun(name)
+    """Convert a plural name to singular form."""
+    singular_name = _get_inflect_engine().singular_noun(cast("inflect.Word", name))
     if singular_name is False:
         singular_name = f"{name}{suffix}"
     return singular_name  # pyright: ignore[reportReturnType]
@@ -680,6 +965,7 @@ def get_singular_name(name: str, suffix: str = SINGULAR_NAME_SUFFIX) -> str:
 
 @lru_cache
 def snake_to_upper_camel(word: str, delimiter: str = "_") -> str:
+    """Convert snake_case or delimited string to UpperCamelCase."""
     prefix = ""
     if word.startswith(delimiter):
         prefix = "_"
@@ -689,7 +975,5 @@ def snake_to_upper_camel(word: str, delimiter: str = "_") -> str:
 
 
 def is_url(ref: str) -> bool:
-    return ref.startswith(("https://", "http://"))
-
-
-inflect_engine = inflect.engine()
+    """Check if a reference string is a URL (HTTP, HTTPS, or file scheme)."""
+    return ref.startswith(("https://", "http://", "file://"))
