@@ -1,3 +1,10 @@
+"""Core type system for data model generation.
+
+Provides DataType for representing types with references and constraints,
+DataTypeManager as the abstract base for type mappings, and supporting
+utilities for handling unions, optionals, and type hints.
+"""
+
 from __future__ import annotations
 
 import re
@@ -21,6 +28,7 @@ from typing import (
 import pydantic
 from packaging import version
 from pydantic import StrictBool, StrictInt, StrictStr, create_model
+from typing_extensions import TypeIs
 
 from datamodel_code_generator.format import (
     DatetimeClassType,
@@ -31,6 +39,7 @@ from datamodel_code_generator.imports import (
     IMPORT_ABC_MAPPING,
     IMPORT_ABC_SEQUENCE,
     IMPORT_ABC_SET,
+    IMPORT_ANY,
     IMPORT_DICT,
     IMPORT_FROZEN_SET,
     IMPORT_LIST,
@@ -45,15 +54,8 @@ from datamodel_code_generator.imports import (
 from datamodel_code_generator.reference import Reference, _BaseModel
 from datamodel_code_generator.util import PYDANTIC_V2, ConfigDict
 
-if TYPE_CHECKING:
-    import builtins
-    from collections.abc import Iterable, Iterator, Sequence
-
-if PYDANTIC_V2:
-    from pydantic import GetCoreSchemaHandler
-    from pydantic_core import core_schema
-
 T = TypeVar("T")
+SourceT = TypeVar("SourceT")
 
 OPTIONAL = "Optional"
 OPTIONAL_PREFIX = f"{OPTIONAL}["
@@ -81,8 +83,22 @@ STR = "str"
 NOT_REQUIRED = "NotRequired"
 NOT_REQUIRED_PREFIX = f"{NOT_REQUIRED}["
 
+if TYPE_CHECKING:
+    import builtins
+    from collections.abc import Iterable, Iterator, Sequence
+
+    from pydantic_core import core_schema
+
+    from datamodel_code_generator.model.base import DataModelFieldBase
+
+if PYDANTIC_V2:
+    from pydantic import GetCoreSchemaHandler
+    from pydantic_core import core_schema
+
 
 class StrictTypes(Enum):
+    """Strict type options for generated models."""
+
     str = "str"
     bytes = "bytes"
     int = "int"
@@ -91,51 +107,52 @@ class StrictTypes(Enum):
 
 
 class UnionIntFloat:
+    """Pydantic-compatible type that accepts both int and float values."""
+
     def __init__(self, value: float) -> None:
+        """Initialize with an int or float value."""
         self.value: int | float = value
 
     def __int__(self) -> int:
+        """Convert value to int."""
         return int(self.value)
 
     def __float__(self) -> float:
+        """Convert value to float."""
         return float(self.value)
 
     def __str__(self) -> str:
+        """Convert value to string."""
         return str(self.value)
 
     @classmethod
     def __get_validators__(cls) -> Iterator[Callable[[Any], Any]]:  # noqa: PLW3201
+        """Return Pydantic v1 validators."""
         yield cls.validate
 
     @classmethod
     def __get_pydantic_core_schema__(  # noqa: PLW3201
         cls, _source_type: Any, _handler: GetCoreSchemaHandler
     ) -> core_schema.CoreSchema:
-        from_int_schema = core_schema.chain_schema(  # pyright: ignore[reportPossiblyUnboundVariable]
-            [
-                core_schema.union_schema(  # pyright: ignore[reportPossiblyUnboundVariable]
-                    [core_schema.int_schema(), core_schema.float_schema()]  # pyright: ignore[reportPossiblyUnboundVariable]
-                ),
-                core_schema.no_info_plain_validator_function(cls.validate),  # pyright: ignore[reportPossiblyUnboundVariable]
-            ]
-        )
+        """Return Pydantic v2 core schema."""
+        from_int_schema = core_schema.chain_schema([
+            core_schema.union_schema([core_schema.int_schema(), core_schema.float_schema()]),
+            core_schema.no_info_plain_validator_function(cls.validate),
+        ])
 
-        return core_schema.json_or_python_schema(  # pyright: ignore[reportPossiblyUnboundVariable]
+        return core_schema.json_or_python_schema(
             json_schema=from_int_schema,
-            python_schema=core_schema.union_schema(  # pyright: ignore[reportPossiblyUnboundVariable]
-                [
-                    # check if it's an instance first before doing any further work
-                    core_schema.is_instance_schema(UnionIntFloat),  # pyright: ignore[reportPossiblyUnboundVariable]
-                    from_int_schema,
-                ]
-            ),
-            serialization=core_schema.plain_serializer_function_ser_schema(  # pyright: ignore[reportPossiblyUnboundVariable]
-                lambda instance: instance.value
-            ),
+            python_schema=core_schema.union_schema([
+                # check if it's an instance first before doing any further work
+                core_schema.is_instance_schema(UnionIntFloat),
+                from_int_schema,
+            ]),
+            serialization=core_schema.plain_serializer_function_ser_schema(lambda instance: instance.value),
         )
 
     @classmethod
     def validate(cls, v: Any) -> UnionIntFloat:
+        """Validate and convert value to UnionIntFloat."""
         if isinstance(v, UnionIntFloat):
             return v
         if not isinstance(v, (int, float)):  # pragma: no cover
@@ -156,32 +173,12 @@ class UnionIntFloat:
 
 
 def chain_as_tuple(*iterables: Iterable[T]) -> tuple[T, ...]:
+    """Chain multiple iterables and return as a tuple."""
     return tuple(chain(*iterables))
 
 
-@lru_cache
-def _remove_none_from_type(type_: str, split_pattern: Pattern[str], delimiter: str) -> list[str]:
-    types: list[str] = []
-    split_type: str = ""
-    inner_count: int = 0
-    for part in re.split(split_pattern, type_):
-        if part == NONE:
-            continue
-        inner_count += part.count("[") - part.count("]")
-        if split_type:
-            split_type += delimiter
-        if inner_count == 0:
-            if split_type:
-                types.append(f"{split_type}{part}")
-            else:
-                types.append(part)
-            split_type = ""
-            continue
-        split_type += part
-    return types
-
-
 def _remove_none_from_union(type_: str, *, use_union_operator: bool) -> str:  # noqa: PLR0912
+    """Remove None from a Union type string, handling nested unions."""
     if use_union_operator:
         if " | " not in type_:
             return type_
@@ -248,6 +245,7 @@ def _remove_none_from_union(type_: str, *, use_union_operator: bool) -> str:  #
 
 @lru_cache
 def get_optional_type(type_: str, use_union_operator: bool) -> str:  # noqa: FBT001
+    """Wrap a type string in Optional or add | None suffix."""
     type_ = _remove_none_from_union(type_, use_union_operator=use_union_operator)
 
     if not type_ or type_ == NONE:
@@ -257,21 +255,36 @@ def get_optional_type(type_: str, use_union_operator: bool) -> str:  # noqa: FBT
     return f"{OPTIONAL_PREFIX}{type_}]"
 
 
+def is_data_model_field(obj: object) -> TypeIs[DataModelFieldBase]:
+    """Check if an object is a DataModelFieldBase instance."""
+    from datamodel_code_generator.model.base import DataModelFieldBase  # noqa: PLC0415
+
+    return isinstance(obj, DataModelFieldBase)
+
+
 @runtime_checkable
 class Modular(Protocol):
+    """Protocol for objects with a module name property."""
+
     @property
     def module_name(self) -> str:
+        """Return the module name."""
         raise NotImplementedError
 
 
 @runtime_checkable
 class Nullable(Protocol):
+    """Protocol for objects with a nullable property."""
+
     @property
     def nullable(self) -> bool:
+        """Return whether the type is nullable."""
         raise NotImplementedError
 
 
 class DataType(_BaseModel):
+    """Represents a type in generated code with imports and references."""
+
     if PYDANTIC_V2:
         # TODO[pydantic]: The following keys were removed: `copy_on_model_validation`.
         # Check https://docs.pydantic.dev/dev-v2/migration/#changes-to-config for more information.
@@ -283,10 +296,18 @@ class DataType(_BaseModel):
         if not TYPE_CHECKING:
 
             @classmethod
-            def model_rebuild(cls) -> None:
-                cls.update_forward_refs()
+            def model_rebuild(
+                cls,
+                *,
+                _types_namespace: dict[str, type] | None = None,
+            ) -> None:
+                """Update forward references for Pydantic v1."""
+                localns = _types_namespace or {}
+                cls.update_forward_refs(**localns)
 
         class Config:
+            """Pydantic v1 model configuration."""
+
             extra = "forbid"
             copy_on_model_validation = False if version.parse(pydantic.VERSION) < version.parse("1.9.2") else "none"
 
@@ -303,15 +324,17 @@ class DataType(_BaseModel):
     is_set: bool = False
     is_custom_type: bool = False
     literals: list[Union[StrictBool, StrictInt, StrictStr]] = []  # noqa: RUF012, UP007
+    enum_member_literals: list[tuple[str, str]] = []  # noqa: RUF012  # [(EnumClassName, member_name), ...]
     use_standard_collections: bool = False
     use_generic_container: bool = False
     use_union_operator: bool = False
     alias: Optional[str] = None  # noqa: UP045
-    parent: Optional[Any] = None  # noqa: UP045
-    children: list[Any] = []  # noqa: RUF012
+    parent: Union[DataModelFieldBase, DataType, None] = None  # noqa: UP007
+    children: list[DataType] = []  # noqa: RUF012
     strict: bool = False
     dict_key: Optional[DataType] = None  # noqa: UP045
     treat_dot_as_module: bool = False
+    use_serialize_as_any: bool = False
 
     _exclude_fields: ClassVar[set[str]] = {"parent", "children"}
     _pass_fields: ClassVar[set[str]] = {"parent", "children", "data_types", "reference"}
@@ -329,6 +352,7 @@ class DataType(_BaseModel):
         strict: bool = False,
         kwargs: dict[str, Any] | None = None,
     ) -> DataTypeT:
+        """Create a DataType from an Import object."""
         return cls(
             type=import_.import_,
             import_=import_,
@@ -344,12 +368,14 @@ class DataType(_BaseModel):
 
     @property
     def unresolved_types(self) -> frozenset[str]:
+        """Return set of unresolved type reference paths."""
         return frozenset(
             {t.reference.path for data_types in self.data_types for t in data_types.all_data_types if t.reference}
             | ({self.reference.path} if self.reference else set())
         )
 
     def replace_reference(self, reference: Reference | None) -> None:
+        """Replace this DataType's reference with a new one."""
         if not self.reference:  # pragma: no cover
             msg = f"`{self.__class__.__name__}.replace_reference()` can't be called when `reference` field is empty."
             raise Exception(msg)  # noqa: TRY002
@@ -360,16 +386,34 @@ class DataType(_BaseModel):
             reference.children.append(self)
 
     def remove_reference(self) -> None:
+        """Remove the reference from this DataType."""
         self.replace_reference(None)
 
+    def swap_with(self, new_data_type: DataType) -> None:
+        """Detach self and attach new_data_type to the same parent.
+
+        Replaces this DataType with new_data_type in the parent container.
+        Works with both field parents and nested DataType parents.
+        """
+        parent = self.parent
+        self.parent = None
+        if parent is not None:  # pragma: no cover
+            new_data_type.parent = parent
+            if is_data_model_field(parent):
+                parent.data_type = new_data_type
+            elif isinstance(parent, DataType):  # pragma: no cover
+                parent.data_types = [new_data_type if d is self else d for d in parent.data_types]
+
     @property
     def module_name(self) -> str | None:
+        """Return the module name from the reference source."""
         if self.reference and isinstance(self.reference.source, Modular):
             return self.reference.source.module_name
         return None  # pragma: no cover
 
     @property
     def full_name(self) -> str:
+        """Return the fully qualified name including module."""
         module_name = self.module_name
         if module_name:
             return f"{module_name}.{self.reference.short_name if self.reference else ''}"
@@ -377,18 +421,31 @@ class DataType(_BaseModel):
 
     @property
     def all_data_types(self) -> Iterator[DataType]:
+        """Recursively yield all nested DataTypes including self."""
         for data_type in self.data_types:
             yield from data_type.all_data_types
         yield self
 
+    def find_source(self, source_type: type[SourceT]) -> SourceT | None:
+        """Find the first reference source matching the given type from all nested data types."""
+        for data_type in self.all_data_types:  # pragma: no branch
+            if not data_type.reference:  # pragma: no cover
+                continue
+            source = data_type.reference.source
+            if isinstance(source, source_type):  # pragma: no cover
+                return source
+        return None  # pragma: no cover
+
     @property
     def all_imports(self) -> Iterator[Import]:
+        """Recursively yield all imports from nested DataTypes and self."""
         for data_type in self.data_types:
             yield from data_type.all_imports
         yield from self.imports
 
     @property
     def imports(self) -> Iterator[Import]:
+        """Yield imports required by this DataType."""
         # Add base import if exists
         if self.import_:
             yield self.import_
@@ -397,7 +454,7 @@ class DataType(_BaseModel):
         imports: tuple[tuple[bool, Import], ...] = (
             (self.is_optional and not self.use_union_operator, IMPORT_OPTIONAL),
             (len(self.data_types) > 1 and not self.use_union_operator, IMPORT_UNION),
-            (bool(self.literals), IMPORT_LITERAL),
+            (bool(self.literals) or bool(self.enum_member_literals), IMPORT_LITERAL),
         )
 
         if self.use_generic_container:
@@ -433,6 +490,7 @@ class DataType(_BaseModel):
             yield from self.dict_key.imports
 
     def __init__(self, **values: Any) -> None:
+        """Initialize DataType with validation and reference setup."""
         if not TYPE_CHECKING:
             super().__init__(**values)
 
@@ -450,15 +508,27 @@ class DataType(_BaseModel):
         if self.reference:
             self.reference.children.append(self)
 
+    def _get_wrapped_reference_type_hint(self, type_: str) -> str:  # noqa: PLR6301
+        """Wrap reference type name if needed (override in subclasses, e.g., for SerializeAsAny).
+
+        Args:
+            type_: The reference type name (e.g., "User")
+
+        Returns:
+            The potentially wrapped type name
+        """
+        return type_
+
     @property
     def type_hint(self) -> str:  # noqa: PLR0912, PLR0915
+        """Generate the Python type hint string for this DataType."""
         type_: str | None = self.alias or self.type
         if not type_:
             if self.is_union:
                 data_types: list[str] = []
                 for data_type in self.data_types:
                     data_type_type = data_type.type_hint
-                    if data_type_type in data_types:  # pragma: no cover
+                    if not data_type_type or data_type_type in data_types:
                         continue
 
                     if data_type_type == NONE:
@@ -473,7 +543,10 @@ class DataType(_BaseModel):
                         self.is_optional = True
 
                     data_types.append(non_optional_data_type_type)
-                if len(data_types) == 1:
+                if not data_types:
+                    type_ = ANY
+                    self.import_ = self.import_ or IMPORT_ANY
+                elif len(data_types) == 1:
                     type_ = data_types[0]
                 elif self.use_union_operator:
                     type_ = UNION_OPERATOR_DELIMITER.join(data_types)
@@ -481,10 +554,14 @@ class DataType(_BaseModel):
                     type_ = f"{UNION_PREFIX}{UNION_DELIMITER.join(data_types)}]"
             elif len(self.data_types) == 1:
                 type_ = self.data_types[0].type_hint
+            elif self.enum_member_literals:
+                parts = [f"{enum_class}.{member}" for enum_class, member in self.enum_member_literals]
+                type_ = f"{LITERAL}[{', '.join(parts)}]"
             elif self.literals:
                 type_ = f"{LITERAL}[{', '.join(repr(literal) for literal in self.literals)}]"
             elif self.reference:
                 type_ = self.reference.short_name
+                type_ = self._get_wrapped_reference_type_hint(type_)
             else:
                 # TODO support strict Any
                 type_ = ""
@@ -531,19 +608,20 @@ class DataType(_BaseModel):
 
     @property
     def is_union(self) -> bool:
+        """Return whether this DataType represents a union of multiple types."""
         return len(self.data_types) > 1
 
 
-DataType.model_rebuild()
-
 DataTypeT = TypeVar("DataTypeT", bound=DataType)
 
 
 class EmptyDataType(DataType):
-    pass
+    """A DataType placeholder for empty or unresolved types."""
 
 
 class Types(Enum):
+    """Standard type identifiers for schema type mapping."""
+
     integer = auto()
     int32 = auto()
     int64 = auto()
@@ -581,6 +659,11 @@ class Types(Enum):
 
 
 class DataTypeManager(ABC):
+    """Abstract base class for managing type mappings in code generation.
+
+    Subclasses implement get_data_type() to map schema types to DataType objects.
+    """
+
     def __init__(  # noqa: PLR0913, PLR0917
         self,
         python_version: PythonVersion = PythonVersionMin,
@@ -588,11 +671,14 @@ class DataTypeManager(ABC):
         use_generic_container_types: bool = False,  # noqa: FBT001, FBT002
         strict_types: Sequence[StrictTypes] | None = None,
         use_non_positive_negative_number_constrained_types: bool = False,  # noqa: FBT001, FBT002
+        use_decimal_for_multiple_of: bool = False,  # noqa: FBT001, FBT002
         use_union_operator: bool = False,  # noqa: FBT001, FBT002
         use_pendulum: bool = False,  # noqa: FBT001, FBT002
         target_datetime_class: DatetimeClassType | None = None,
         treat_dot_as_module: bool = False,  # noqa: FBT001, FBT002
+        use_serialize_as_any: bool = False,  # noqa: FBT001, FBT002
     ) -> None:
+        """Initialize DataTypeManager with code generation options."""
         self.python_version = python_version
         self.use_standard_collections: bool = use_standard_collections
         self.use_generic_container_types: bool = use_generic_container_types
@@ -600,32 +686,35 @@ class DataTypeManager(ABC):
         self.use_non_positive_negative_number_constrained_types: bool = (
             use_non_positive_negative_number_constrained_types
         )
+        self.use_decimal_for_multiple_of: bool = use_decimal_for_multiple_of
         self.use_union_operator: bool = use_union_operator
         self.use_pendulum: bool = use_pendulum
         self.target_datetime_class: DatetimeClassType | None = target_datetime_class
         self.treat_dot_as_module: bool = treat_dot_as_module
+        self.use_serialize_as_any: bool = use_serialize_as_any
 
-        if TYPE_CHECKING:
-            self.data_type: type[DataType]
-        else:
-            self.data_type: type[DataType] = create_model(
-                "ContextDataType",
-                python_version=(PythonVersion, python_version),
-                use_standard_collections=(bool, use_standard_collections),
-                use_generic_container=(bool, use_generic_container_types),
-                use_union_operator=(bool, use_union_operator),
-                treat_dot_as_module=(bool, treat_dot_as_module),
-                __base__=DataType,
-            )
+        self.data_type: type[DataType] = create_model(
+            "ContextDataType",
+            python_version=(PythonVersion, python_version),
+            use_standard_collections=(bool, use_standard_collections),
+            use_generic_container=(bool, use_generic_container_types),
+            use_union_operator=(bool, use_union_operator),
+            treat_dot_as_module=(bool, treat_dot_as_module),
+            use_serialize_as_any=(bool, use_serialize_as_any),
+            __base__=DataType,
+        )
 
     @abstractmethod
     def get_data_type(self, types: Types, **kwargs: Any) -> DataType:
+        """Map a Types enum value to a DataType. Must be implemented by subclasses."""
         raise NotImplementedError
 
     def get_data_type_from_full_path(self, full_path: str, is_custom_type: bool) -> DataType:  # noqa: FBT001
+        """Create a DataType from a fully qualified Python path."""
         return self.data_type.from_import(Import.from_full_path(full_path), is_custom_type=is_custom_type)
 
     def get_data_type_from_value(self, value: Any) -> DataType:
+        """Infer a DataType from a Python value."""
         type_: Types | None = None
         if isinstance(value, str):
             type_ = Types.string
