@@ -4,11 +4,12 @@ from typing import Optional
 
 from pydantic import BaseModel
 
-from .. import Source, bar, foo
+from .. import bar
+from .._internal import Cocoa, Source
 
 
 class Chocolate(BaseModel):
     flavour: Optional[str] = None
     source: Optional[Source] = None
-    cocoa: Optional[foo.Cocoa] = None
+    cocoa: Optional[Cocoa] = None
     field: Optional[bar.Field] = None
