@@ -1,3 +1,5 @@
+"""Tests for Pydantic constraint detection."""
+
 from __future__ import annotations
 
 import pytest
@@ -16,6 +18,7 @@ from datamodel_code_generator.types import UnionIntFloat
     ],
 )
 def test_constraint(gt: float | None, expected: bool) -> None:
+    """Test constraint detection with greater-than values."""
     constraints = Constraints()
     constraints.gt = UnionIntFloat(gt) if gt is not None else None
     assert constraints.has_constraints == expected
