@@ -1,3 +1,5 @@
+"""Tests for OpenAPI/Swagger schema parser."""
+
 from __future__ import annotations
 
 import os
@@ -22,6 +24,7 @@ from datamodel_code_generator.parser.openapi import (
     RequestBodyObject,
     ResponseObject,
 )
+from tests.conftest import assert_output, assert_parser_modules, assert_parser_results
 
 DATA_PATH: Path = Path(__file__).parents[1] / "data" / "openapi"
 
@@ -35,6 +38,7 @@ def get_expected_file(
     base_class: str | None = None,
     prefix: str | None = None,
 ) -> Path:
+    """Get expected output file path for test."""
     params: list[str] = []
     if with_import:
         params.append("with_import")
@@ -127,11 +131,12 @@ class Pets(BaseModel):
         (
             {"properties": {"kind": {"type": "array", "items": []}}},
             """class Pets(BaseModel):
-    kind: Optional[List] = None""",
+    kind: Optional[List[Any]] = None""",
         ),
     ],
 )
 def test_parse_object(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI object schemas."""
     parser = OpenAPIParser("")
     parser.parse_object("Pets", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
@@ -170,11 +175,12 @@ class Pets(BaseModel):
                 "items": {},
             },
             """class Pets(BaseModel):
-    __root__: List""",
+    __root__: List[Any]""",
         ),
     ],
 )
 def test_parse_array(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI array schemas."""
     parser = OpenAPIParser("")
     parser.parse_array("Pets", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
@@ -202,16 +208,14 @@ def test_parse_array(source_obj: dict[str, Any], generated_classes: str) -> None
     ],
 )
 def test_openapi_parser_parse(with_import: bool, format_: bool, base_class: str | None) -> None:
+    """Test OpenAPI parser with various configurations."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "api.yaml"),
         base_class=base_class,
     )
     expected_file = get_expected_file("openapi_parser_parse", with_import, format_, base_class)
-    assert (
-        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent)
-        == expected_file.read_text()
-    )
+    assert_output(parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent), expected_file)
 
 
 @pytest.mark.parametrize(
@@ -230,39 +234,42 @@ def test_openapi_parser_parse(with_import: bool, format_: bool, base_class: str
     ],
 )
 def test_parse_root_type(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI root type schemas."""
     parser = OpenAPIParser("")
     parser.parse_root_type("Name", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
 
 
 def test_openapi_parser_parse_duplicate_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with duplicate model names."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "duplicate_models.yaml"),
     )
-    assert (
-        parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models" / "output.py").read_text()
-    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models" / "output.py")
 
 
 def test_openapi_parser_parse_duplicate_model_with_simplify(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with duplicate models and simplification."""
     monkeypatch.chdir(tmp_path)
     raw = Path(DATA_PATH / "duplicate_model_simplify.yaml")
     parser = OpenAPIParser(raw)
-    expected = (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models_simplify" / "output.py").read_text()
-    got = parser.parse()
-    assert got == expected
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models_simplify" / "output.py"
+    )
 
 
 def test_openapi_parser_parse_resolved_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with resolved model references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "resolved_models.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_resolved_models" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_resolved_models" / "output.py")
 
 
 def test_openapi_parser_parse_lazy_resolved_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with lazy resolved model references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "lazy_resolved_models.yaml"),
@@ -308,6 +315,7 @@ class Results(BaseModel):
 
 
 def test_openapi_parser_parse_x_enum_varnames(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with x-enum-varnames extension."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "x_enum_varnames.yaml"),
@@ -362,106 +370,106 @@ class UnknownTypeNumber(Enum):
 
 @pytest.mark.skipif(pydantic.VERSION < "1.9.0", reason="Require Pydantic version 1.9.0 or later ")
 def test_openapi_parser_parse_enum_models() -> None:
+    """Test parsing OpenAPI enum models."""
     parser = OpenAPIParser(
         Path(DATA_PATH / "enum_models.yaml").read_text(encoding="utf-8"),
         target_python_version=PythonVersionMin,
     )
     expected_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_enum_models"
-    assert parser.parse() == (expected_dir / "output.py").read_text()
+    assert_output(parser.parse(), expected_dir / "output.py")
 
 
 def test_openapi_parser_parse_anyof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with anyOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "anyof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof" / "output.py")
 
 
 def test_openapi_parser_parse_anyof_required(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with anyOf and required fields."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "anyof_required.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof_required" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof_required" / "output.py")
 
 
 def test_openapi_parser_parse_nested_anyof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with nested anyOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "nested_anyof.yaml").read_text(encoding="utf-8"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_anyof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_anyof" / "output.py")
 
 
 def test_openapi_parser_parse_oneof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with oneOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "oneof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_oneof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_oneof" / "output.py")
 
 
 def test_openapi_parser_parse_nested_oneof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with nested oneOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "nested_oneof.yaml").read_text(encoding="utf-8"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_oneof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_oneof" / "output.py")
 
 
 def test_openapi_parser_parse_allof_ref(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "allof_same_prefix_with_ref.yaml"),
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_same_prefix_with_ref" / "output.py").read_text()
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_same_prefix_with_ref" / "output.py"
     )
 
 
 def test_openapi_parser_parse_allof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "allof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof" / "output.py")
 
 
 def test_openapi_parser_parse_allof_required_fields(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf and required fields."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "allof_required_fields.yaml"),
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_required_fields" / "output.py").read_text()
-    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_required_fields" / "output.py")
 
 
 def test_openapi_parser_parse_alias(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with field aliases."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "alias.yaml"),
     )
     delimiter = "\\" if platform.system() == "Windows" else "/"
     results = {delimiter.join(p): r for p, r in parser.parse().items()}
-    openapi_parser_parse_alias_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_alias"
-    for path in openapi_parser_parse_alias_dir.rglob("*.py"):
-        key = str(path.relative_to(openapi_parser_parse_alias_dir))
-        assert results.pop(key).body == path.read_text()
+    assert_parser_results(results, EXPECTED_OPEN_API_PATH / "openapi_parser_parse_alias")
 
 
 def test_openapi_parser_parse_modular(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with modular structure."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(Path(DATA_PATH / "modular.yaml"), data_model_field_type=DataModelFieldBase)
     modules = parser.parse()
-    main_modular_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_modular"
-
-    for paths, result in modules.items():
-        expected = main_modular_dir.joinpath(*paths).read_text()
-        assert result.body == expected
+    assert_parser_modules(modules, EXPECTED_OPEN_API_PATH / "openapi_parser_parse_modular")
 
 
 @pytest.mark.parametrize(
@@ -490,32 +498,68 @@ def test_openapi_parser_parse_modular(tmp_path: Path, monkeypatch: pytest.Monkey
     ],
 )
 def test_openapi_parser_parse_additional_properties(with_import: bool, format_: bool, base_class: str | None) -> None:
+    """Test parsing OpenAPI with additional properties."""
     parser = OpenAPIParser(
         Path(DATA_PATH / "additional_properties.yaml").read_text(encoding="utf-8"),
         base_class=base_class,
         data_model_field_type=DataModelFieldBase,
     )
 
-    assert (
-        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent)
-        == get_expected_file(
+    assert_output(
+        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent),
+        get_expected_file(
             "openapi_parser_parse_additional_properties",
             with_import,
             format_,
             base_class,
-        ).read_text()
+        ),
     )
 
 
 def test_openapi_parser_parse_array_enum(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with array enum types."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(source=Path(DATA_PATH / "array_enum.yaml"))
     expected_file = get_expected_file("openapi_parser_parse_array_enum", True, True)
-    assert parser.parse() == expected_file.read_text()
+    assert_output(parser.parse(), expected_file)
 
 
-def test_openapi_parser_parse_remote_ref(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+def test_openapi_parser_parse_remote_ref(tmp_path: Path, monkeypatch: pytest.MonkeyPatch, mocker: Any) -> None:
+    """Test parsing OpenAPI with remote references."""
     monkeypatch.chdir(tmp_path)
+
+    remote_schema = """
+schemas:
+  Problem:
+    properties:
+      detail:
+        description: A human readable explanation specific to this occurrence of the problem.
+        type: string
+      instance:
+        description: An absolute URI that identifies the specific occurrence of the problem.
+        format: uri
+        type: string
+      status:
+        description: The HTTP status code generated by the origin server for this occurrence of the problem.
+        exclusiveMaximum: true
+        format: int32
+        maximum: 600
+        minimum: 100
+        type: integer
+      title:
+        description: A short, summary of the problem type.
+        type: string
+      type:
+        default: about:blank
+        description: An absolute URI that identifies the problem type.
+        format: uri
+        type: string
+    type: object
+"""
+    mock_response = mocker.Mock()
+    mock_response.text = remote_schema
+    mocker.patch("httpx.get", return_value=mock_response)
+
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=(DATA_PATH / "refs.yaml").read_text(),
@@ -523,16 +567,18 @@ def test_openapi_parser_parse_remote_ref(tmp_path: Path, monkeypatch: pytest.Mon
     )
     expected_file = get_expected_file("openapi_parser_parse_remote_ref", True, True)
 
-    assert parser.parse() == expected_file.read_text()
+    assert_output(parser.parse(), expected_file)
 
 
 def test_openapi_parser_parse_required_null(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with required nullable fields."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(source=Path(DATA_PATH / "required_null.yaml"))
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_required_null" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_required_null" / "output.py")
 
 
 def test_openapi_model_resolver(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test OpenAPI model resolver functionality."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(source=(DATA_PATH / "api.yaml"))
     parser.parse()
@@ -625,15 +671,17 @@ def test_openapi_model_resolver(tmp_path: Path, monkeypatch: pytest.MonkeyPatch)
 
 
 def test_openapi_parser_parse_any(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with any type schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "any.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_any" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_any" / "output.py")
 
 
 def test_openapi_parser_responses_without_content(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI responses without content."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
@@ -641,20 +689,18 @@ def test_openapi_parser_responses_without_content(tmp_path: Path, monkeypatch: p
         openapi_scopes=[OpenAPIScope.Paths],
         allow_responses_without_content=True,
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_responses_without_content" / "output.py").read_text()
-    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_responses_without_content" / "output.py")
 
 
 def test_openapi_parser_responses_with_tag(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI responses with tags."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "body_and_parameters.yaml"),
         openapi_scopes=[OpenAPIScope.Tags, OpenAPIScope.Schemas, OpenAPIScope.Paths],
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_responses_with_tag" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_responses_with_tag" / "output.py")
 
 
 @pytest.mark.skipif(
@@ -662,6 +708,7 @@ def test_openapi_parser_responses_with_tag(tmp_path: Path, monkeypatch: pytest.M
     reason="Installed black doesn't support the old style",
 )
 def test_openapi_parser_with_query_parameters() -> None:
+    """Test parsing OpenAPI with query parameters."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "query_parameters.yaml"),
@@ -671,7 +718,7 @@ def test_openapi_parser_with_query_parameters() -> None:
             OpenAPIScope.Paths,
         ],
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "output.py")
 
 
 @pytest.mark.skipif(
@@ -679,6 +726,7 @@ def test_openapi_parser_with_query_parameters() -> None:
     reason="Installed black doesn't support the old style",
 )
 def test_openapi_parser_with_include_path_parameters() -> None:
+    """Test parsing OpenAPI with included path parameters."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "query_parameters.yaml"),
@@ -689,13 +737,13 @@ def test_openapi_parser_with_include_path_parameters() -> None:
         ],
         include_path_parameters=True,
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "with_path_params.py").read_text()
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "with_path_params.py"
     )
 
 
 def test_parse_all_parameters_duplicate_names_exception() -> None:
+    """Test parsing parameters with duplicate names raises exception."""
     parser = OpenAPIParser("", include_path_parameters=True)
     parameters = [
         ParameterObject.parse_obj({"name": "duplicate_param", "in": "path", "schema": {"type": "string"}}),
@@ -713,6 +761,7 @@ def test_parse_all_parameters_duplicate_names_exception() -> None:
     reason="Require Pydantic version 2.0.0 or later ",
 )
 def test_openapi_parser_array_called_fields_with_one_of_items() -> None:
+    """Test parsing OpenAPI array fields with oneOf items."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelField,
         source=Path(DATA_PATH / "array_called_fields_with_oneOf_items.yaml"),
@@ -723,11 +772,9 @@ def test_openapi_parser_array_called_fields_with_one_of_items() -> None:
         ],
         field_constraints=True,
     )
-    assert (
-        parser.parse()
-        == (
-            EXPECTED_OPEN_API_PATH / "openapi_parser_parse_array_called_fields_with_oneOf_items" / "output.py"
-        ).read_text()
+    assert_output(
+        parser.parse(),
+        EXPECTED_OPEN_API_PATH / "openapi_parser_parse_array_called_fields_with_oneOf_items" / "output.py",
     )
 
 
@@ -775,6 +822,7 @@ def test_no_additional_imports() -> None:
     ],
 )
 def test_parse_request_body_return(request_body_data: dict[str, Any], expected_type_hints: dict[str, str]) -> None:
+    """Test parsing request body returns correct type hints."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source="",
@@ -814,6 +862,7 @@ def test_parse_request_body_return(request_body_data: dict[str, Any], expected_t
     ],
 )
 def test_parse_all_parameters_return(parameters_data: list[dict[str, Any]], expected_type_hint: str | None) -> None:
+    """Test parsing parameters returns correct type hints."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source="",
@@ -877,6 +926,7 @@ def test_parse_responses_return(
     responses_data: dict[str, dict[str, Any]],
     expected_type_hints: dict[str, dict[str, str]],
 ) -> None:
+    """Test parsing responses returns correct type hints."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source="",
